"""
Modular symbols using eclib newforms
"""

# ****************************************************************************
#       Copyright (C) 2008 Tom Boothby <boothby@u.washington.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from cysignals.signals cimport sig_on, sig_off

from sage.libs.eclib cimport *
from sage.libs.gmp.mpq cimport mpq_numref
from sage.libs.ntl.convert cimport mpz_to_ZZ
from sage.rings.rational_field import QQ
from sage.rings.rational cimport Rational
from sage.modular.all import Cusp


cdef class ECModularSymbol:
    r"""
    Modular symbol associated with an elliptic curve,  using John Cremona's newforms class.

    EXAMPLES::

        sage: from sage.libs.eclib.newforms import ECModularSymbol
        sage: E = EllipticCurve('11a')
        sage: M = ECModularSymbol(E,1); M
        Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field

    By default, symbols are based at the cusp `\infty`, i.e. we evaluate `\{\infty,r\}`::

        sage: [M(1/i) for i in range(1,11)]
        [2/5, -8/5, -3/5, 7/5, 12/5, 12/5, 7/5, -3/5, -8/5, 2/5]

    We can also switch the base point to the cusp `0`::

        sage: [M(1/i, base_at_infinity=False) for i in range(1,11)]
        [0, -2, -1, 1, 2, 2, 1, -1, -2, 0]

    For the minus symbols this makes no difference since
    `\{0,\infty\}` is in the plus space.  Note that to evaluate minus
    symbols the space must be defined with sign 0, which makes both
    signs available::

        sage: M = ECModularSymbol(E,0); M
        Modular symbol with sign 0 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field
        sage: [M(1/i, -1) for i in range(1,11)]
        [0, 0, 1, 1, 0, 0, -1, -1, 0, 0]
        sage: [M(1/i, -1, base_at_infinity=False) for i in range(1,11)]
        [0, 0, 1, 1, 0, 0, -1, -1, 0, 0]

    If the ECModularSymbol is created with sign 0 then as well as
    asking for both + and - symbols, we can also obtain both (as a
    tuple).  However it is more work to create the full modular
    symbol space::

        sage: E = EllipticCurve('11a1')
        sage: M = ECModularSymbol(E,0); M
        Modular symbol with sign 0 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field
        sage: [M(1/i) for i in range(2,11)]
        [[-8/5, 0],
         [-3/5, 1],
         [7/5, 1],
         [12/5, 0],
         [12/5, 0],
         [7/5, -1],
         [-3/5, -1],
         [-8/5, 0],
         [2/5, 0]]

    The curve is automatically converted to its minimal model::

        sage: E = EllipticCurve([0,0,0,0,1/4])
        sage: ECModularSymbol(E)
        Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 over Rational Field

    Non-optimal curves are handled correctly in eclib, by comparing the ratios of real and/or imaginary periods::

        sage: from sage.libs.eclib.newforms import ECModularSymbol
        sage: E1 = EllipticCurve('11a1') # optimal
        sage: E1.period_lattice().basis()
        (1.26920930427955, 0.634604652139777 + 1.45881661693850*I)
        sage: M1 = ECModularSymbol(E1,0)
        sage: M1(0)
        [2/5, 0]
        sage: M1(1/3)
        [-3/5, 1]

    One non-optimal curve has real period 1/5 that of the optimal one, so plus symbols scale up by a factor of 5 while minus symbols are unchanged::

        sage: E2 = EllipticCurve('11a2') # not optimal
        sage: E2.period_lattice().basis()
        (0.253841860855911, 0.126920930427955 + 1.45881661693850*I)
        sage: M2 = ECModularSymbol(E2,0)
        sage: M2(0)
        [2, 0]
        sage: M2(1/3)
        [-3, 1]
        sage: all((M2(r,1)==5*M1(r,1)) for r in QQ.range_by_height(10))
        True
        sage: all((M2(r,-1)==M1(r,-1)) for r in QQ.range_by_height(10))
        True

    The other non-optimal curve has real period 5 times that of the optimal one, so plus symbols scale down by a factor of 5; again, minus symbols are unchanged::

        sage: E3 = EllipticCurve('11a3') # not optimal
        sage: E3.period_lattice().basis()
        (6.34604652139777, 3.17302326069888 + 1.45881661693850*I)
        sage: M3 = ECModularSymbol(E3,0)
        sage: M3(0)
        [2/25, 0]
        sage: M3(1/3)
        [-3/25, 1]
        sage: all((5*M3(r,1)==M1(r,1)) for r in QQ.range_by_height(10))
        True
        sage: all((M3(r,-1)==M1(r,-1)) for r in QQ.range_by_height(10))
        True

    TESTS::

        sage: ECModularSymbol.__new__(ECModularSymbol)
        Modular symbol with sign 0 over Rational Field attached to None
    """
    def __init__(self, E, sign=1, nap=1000):
        """
        Construct the modular symbol object from an elliptic curve.

        INPUT:

        - ``E``- an elliptic curve defined over Q.

        - ``sign`` -- integer; 0 or +1.  If +1, only plus modular symbols
         of this sign are available.  If 0, modular symbols of both
         signs are available but the construction is more expensive.

        - ``nap`` -- integer (default: 1000); the number of ap of E to use
         in determining the normalisation of the modular symbols.
         Note that eclib will increase this to 100*sqrt(N) if necessary.

        EXAMPLES::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a')
            sage: M = ECModularSymbol(E, +1)
            sage: M
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field
            sage: E = EllipticCurve('37a')
            sage: M = ECModularSymbol(E, -1)
            Traceback (most recent call last):
            ...
            ValueError: ECModularSymbol can only be created with signs +1 or 0, not -1
            sage: E = EllipticCurve('389a')
            sage: M = ECModularSymbol(E, 0)
            sage: M
            Modular symbol with sign 0 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 + x^2 - 2*x over Rational Field

        TESTS:

        This one is from :issue:`8042`::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('858k2')
            sage: ECModularSymbol(E)
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + x*y = x^3 + 16353089*x - 335543012233 over Rational Field

        We allow a-invariants which are larger than 64 bits
        (:issue:`16977`)::

            sage: E = EllipticCurve([-25194941007454971, -1539281792450963687794218])  # non-minimal model of 21758k3
            sage: ECModularSymbol(E)  # long time
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + x*y = x^3 - 19440540900814*x - 32992152521343165084 over Rational Field

        The curve needs to be defined over the rationals::

            sage: K.<i> = QuadraticField(-1)
            sage: E = EllipticCurve(K, [0,0,0,0,1])
            sage: ECModularSymbol(E)
            Traceback (most recent call last):
            ...
            TypeError: the elliptic curve must have coefficients in the rational numbers
        """
        cdef ZZ_c a1, a2, a3, a4, a6, N
        cdef Curve C
        cdef Curvedata CD
        cdef CurveRed CR
        cdef int n

        if E.base_field() is not QQ:
            raise TypeError("the elliptic curve must have coefficients in the rational numbers")

        E = E.minimal_model()
        self._E = E

        # The a invariants are rational numbers with denominator 1
        mpz_to_ZZ(&a1, mpq_numref((<Rational>(E.a1())).value))
        mpz_to_ZZ(&a2, mpq_numref((<Rational>(E.a2())).value))
        mpz_to_ZZ(&a3, mpq_numref((<Rational>(E.a3())).value))
        mpz_to_ZZ(&a4, mpq_numref((<Rational>(E.a4())).value))
        mpz_to_ZZ(&a6, mpq_numref((<Rational>(E.a6())).value))

        sig_on()
        C = Curve(a1,a2,a3,a4,a6)
        CD = Curvedata(C,0)
        CR = CurveRed(CD)
        N = getconductor(CR)
        n = I2int(N)
        self.n = n
        if not (sign == 0 or sign == 1):
            sig_off()
            raise ValueError("ECModularSymbol can only be created with signs +1 or 0, not {}".format(sign))
        self.sign = sign

        self.nfs = new newforms(n, 0)
        self.nfs.createfromcurve(sign, CR, nap)
        sig_off()

    def __dealloc__(self):
        del self.nfs

    def __repr__(self):
        """
        TESTS::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a')
            sage: M = ECModularSymbol(E); M
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field
            sage: E = EllipticCurve('37a')
            sage: M = ECModularSymbol(E, 1); M
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x over Rational Field
            sage: E = EllipticCurve('389a')
            sage: M = ECModularSymbol(E, 0); M
            Modular symbol with sign 0 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 + x^2 - 2*x over Rational Field
        """
        return "Modular symbol with sign %s over Rational Field attached to %s" % (self.sign, self._E)

    def __call__(self, r, sign=None, base_at_infinity=True):
        r"""
        Compute the value of ``self`` on `\{0,r\}` or `\{\infty, r\}` for
        rational `r`.

        INPUT:

        - ``r`` -- rational; a rational number

        - ``sign`` -- integer; either +1, -1 or 0.  If the sign of the
          space is +1, only sign +1 is allowed.  Default: ``self.sign``, or +1
          when ``self.sign==0``.

        - ``base_at_infinity`` -- boolean; if ``True``, evaluates
          `\{\infty, r\}`. Otherwise (default) evaluates `\{0,r\}`.

        OUTPUT:

        If sign=+1, the rational value of the plus modular symbol is
        returned.  If sign=-1, the rational value of the minus modular
        symbol is returned.  If sign=0, a tuple of both the plus and
        minus modular symbols is returned.  In the last two cases, the
        space must have sign 0 or an error will be raised.

        EXAMPLES::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a')
            sage: M = ECModularSymbol(E)
            sage: [M(1/i) for i in range(1,11)]
            [2/5, -8/5, -3/5, 7/5, 12/5, 12/5, 7/5, -3/5, -8/5, 2/5]
            sage: [M(1/i, base_at_infinity=False) for i in range(1,11)]
            [0, -2, -1, 1, 2, 2, 1, -1, -2, 0]
            sage: E = EllipticCurve('37a')
            sage: M = ECModularSymbol(E)
            sage: [M(1/i) for i in range(1,11)]
            [0, 0, 0, 0, 1, 0, 1, 1, 0, 0]
            sage: E = EllipticCurve('389a')
            sage: M = ECModularSymbol(E)
            sage: [M(1/i) for i in range(1,11)]
            [0, 0, 0, 0, 2, 0, 1, 0, -1, 0]

        When the class is created with sign 0 we can ask for +1 or -1 symbols or (by default) both::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a1')
            sage: M = ECModularSymbol(E,0)
            sage: [M(1/i, 1) for i in range(2,11)]
            [-8/5, -3/5, 7/5, 12/5, 12/5, 7/5, -3/5, -8/5, 2/5]
            sage: [M(1/i, -1) for i in range(2,11)]
            [0, 1, 1, 0, 0, -1, -1, 0, 0]
            sage: [M(1/i) for i in range(2,11)]
            [[-8/5, 0],
             [-3/5, 1],
             [7/5, 1],
             [12/5, 0],
             [12/5, 0],
             [7/5, -1],
             [-3/5, -1],
             [-8/5, 0],
             [2/5, 0]]

        When the class is created with sign +1 we can only ask for +1 symbols::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a1')
            sage: M = ECModularSymbol(E)
            sage: M
            Modular symbol with sign 1 over Rational Field attached to Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field
            sage: M(0)
            2/5
            sage: M(1/3, sign=-1)
            Traceback (most recent call last):
            ...
            ValueError: impossible to evaluate a minus symbol on a plus space
            sage: M(2/3, sign=0)
            Traceback (most recent call last):
            ...
            ValueError: impossible to evaluate both symbols on a plus space

        TESTS:

        See :issue:`11211`::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a')
            sage: M = ECModularSymbol(E)
            sage: M(oo)
            0
            sage: M(oo, base_at_infinity=False)
            -2/5
            sage: M(7/5)
            -13/5
            sage: M("garbage")
            Traceback (most recent call last):
            ...
            TypeError: unable to convert 'garbage' to a cusp
            sage: M(7/5)
            -13/5
        """
        cdef rational _r
        cdef rational _sp, _sm
        cdef pair[rational,rational] _spm
        cdef long n, d

        r = Cusp(r)
        d = r.denominator()
        n = r.numerator()
        if d != 0:
            n = n % d
        sig_on()
        _r = rational(n, d)
        if sign is None or sign not in [-1, 0, 1]:
            sign = self.sign
        if sign == +1:
            _sp = self.nfs.plus_modular_symbol(_r, 0, int(base_at_infinity))
        elif sign == -1:
            if self.sign != 0:
                sig_off()
                raise ValueError("impossible to evaluate a minus symbol on a plus space")
            else:
                _sm = self.nfs.minus_modular_symbol(_r, 0, int(base_at_infinity))
        else:  # sign == 0
            if self.sign != 0:
                sig_off()
                raise ValueError("impossible to evaluate both symbols on a plus space")
            else:
                _spm = self.nfs.full_modular_symbol(_r, 0, int(base_at_infinity))
                _sp = _spm.first
                _sm = _spm.second

        sig_off()

        if sign == +1:
            return Rational((rational_num(_sp), rational_den(_sp)))
        elif sign == -1:
            return Rational((rational_num(_sm), rational_den(_sm)))
        else:
            return [Rational((rational_num(_sp), rational_den(_sp))),
                    Rational((rational_num(_sm), rational_den(_sm)))]

    def __reduce__(self):
        """
        TESTS::

            sage: from sage.libs.eclib.newforms import ECModularSymbol
            sage: E = EllipticCurve('11a')
            sage: M = ECModularSymbol(E)
            sage: M.__reduce__()
            (<class 'sage.libs.eclib.newforms.ECModularSymbol'>,
             (Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20 over Rational Field,
              1))
        """
        return (ECModularSymbol, (self._E, self.sign))



"""
Base class for groups
"""

# ****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.rings.infinity import infinity
from sage.structure.parent cimport Parent


cdef class Group(Parent):
    """
    Base class for all groups.

    TESTS::

        sage: from sage.groups.group import Group
        sage: G = Group()
        sage: TestSuite(G).run(skip = ["_test_an_element",\
        ....:                          "_test_associativity",\
        ....:                          "_test_elements",\
        ....:                          "_test_elements_eq_reflexive",\
        ....:                          "_test_elements_eq_symmetric",\
        ....:                          "_test_elements_eq_transitive",\
        ....:                          "_test_elements_neq",\
        ....:                          "_test_inverse",\
        ....:                          "_test_one",\
        ....:                          "_test_pickling",\
        ....:                          "_test_prod",\
        ....:                          "_test_some_elements"])

    Generic groups have very little functionality::

        sage: 4 in G
        Traceback (most recent call last):
        ...
        NotImplementedError: cannot construct elements of <sage.groups.group.Group object at ...>
    """
    def __init__(self, base=None, category=None):
        """
        The Python constructor.

        TESTS::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.category()
            Category of groups
            sage: G = Group(category=Groups()) # todo: do the same test with some subcategory of Groups when there will exist one
            sage: G.category()
            Category of groups
            sage: G = Group(category=CommutativeAdditiveGroups())
            Traceback (most recent call last):
            ...
            ValueError: (Category of commutative additive groups,) is not a subcategory of Category of groups
            sage: G._repr_option('element_is_atomic')
            False

        Check for :issue:`8119`::

            sage: # needs sage.groups
            sage: G = SymmetricGroup(2)
            sage: h = hash(G)
            sage: G.rename('S2')
            sage: h == hash(G)
            True
        """
        from sage.categories.groups import Groups
        if category is None:
            category = Groups()
        else:
            if not isinstance(category, tuple):
                category = (category,)
            if not any(cat.is_subcategory(Groups()) for cat in category):
                raise ValueError("%s is not a subcategory of %s" % (category, Groups()))
        Parent.__init__(self, base=base, category=category)

    def is_abelian(self) -> bool:
        """
        Test whether this group is abelian.

        EXAMPLES::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.is_abelian()
            Traceback (most recent call last):
            ...
            NotImplementedError
        """
        raise NotImplementedError

    def is_commutative(self) -> bool:
        r"""
        Test whether this group is commutative.

        This is an alias for is_abelian, largely to make groups work
        well with the Factorization class.

        (Note for developers: Derived classes should override is_abelian, not
        is_commutative.)

        EXAMPLES::

            sage: SL(2, 7).is_commutative()                                             # needs sage.libs.gap sage.modules sage.rings.finite_rings
            False
        """
        return self.is_abelian()

    def order(self):
        """
        Return the number of elements of this group.

        This is either a positive integer or infinity.

        EXAMPLES::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.order()
            Traceback (most recent call last):
            ...
            NotImplementedError

        TESTS::

            sage: H = SL(2, QQ)                                                         # needs sage.modules
            sage: H.order()                                                             # needs sage.modules
            +Infinity
        """
        try:
            return self.cardinality()
        except AttributeError:
            raise NotImplementedError

    def is_finite(self):
        """
        Return ``True`` if this group is finite.

        EXAMPLES::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.is_finite()
            Traceback (most recent call last):
            ...
            NotImplementedError
        """
        return self.order() != infinity

    def is_trivial(self):
        r"""
        Return ``True`` if this group is the trivial group.

        A group is trivial, if it consists only of the identity
        element.

        .. WARNING::

            It is in principle undecidable whether a group is
            trivial, for example, if the group is given by a finite
            presentation.  Thus, this method may not terminate.

        EXAMPLES::

            sage: groups.presentation.Cyclic(1).is_trivial()
            True

            sage: G.<a,b> = FreeGroup('a, b')
            sage: H = G / (a^2, b^3, a*b*~a*~b)
            sage: H.is_trivial()
            False

        A non-trivial presentation of the trivial group::

            sage: F.<a,b> = FreeGroup()
            sage: J = F / ((~a)*b*a*(~b)^2, (~b)*a*b*(~a)^2)
            sage: J.is_trivial()
            True
        """
        return self.order() == 1

    def is_multiplicative(self):
        r"""
        Return ``True`` if the group operation is given by ``*`` (rather than ``+``).

        Override for additive groups.

        EXAMPLES::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.is_multiplicative()
            True
        """
        return True

    def _an_element_(self):
        """
        Return an element.

        OUTPUT: an element of the group

        EXAMPLES::

            sage: G = AbelianGroup([2,3,4,5])                                           # needs sage.modules
            sage: G.an_element()                                                        # needs sage.modules
            f0*f1*f2*f3
        """
        return self.prod(self.gens())

    def quotient(self, H, **kwds):
        """
        Return the quotient of this group by the normal subgroup `H`.

        EXAMPLES::

            sage: from sage.groups.group import Group
            sage: G = Group()
            sage: G.quotient(G)
            Traceback (most recent call last):
            ...
            NotImplementedError
        """
        raise NotImplementedError

cdef class AbelianGroup(Group):
    """
    Generic abelian group.
    """
    def is_abelian(self) -> bool:
        """
        Return ``True``.

        EXAMPLES::

            sage: from sage.groups.group import AbelianGroup
            sage: G = AbelianGroup()
            sage: G.is_abelian()
            True
        """
        return True

cdef class FiniteGroup(Group):
    """
    Generic finite group.
    """

    def __init__(self, base=None, category=None):
        """
        The Python constructor.

        TESTS::

            sage: from sage.groups.group import FiniteGroup
            sage: G = FiniteGroup()
            sage: G.category()
            Category of finite groups
        """
        from sage.categories.finite_groups import FiniteGroups
        if category is None:
            category = FiniteGroups()
        else:
            if not isinstance(category, tuple):
                category = (category,)
            if not any(cat.is_subcategory(FiniteGroups()) for cat in category):
                raise ValueError("%s is not a subcategory of %s" % (category, FiniteGroups()))
        Parent.__init__(self, base=base, category=category)

    def is_finite(self):
        """
        Return ``True``.

        EXAMPLES::

            sage: from sage.groups.group import FiniteGroup
            sage: G = FiniteGroup()
            sage: G.is_finite()
            True
        """
        return True



# sage.doctest: needs sage.combinat
"""
Classical Ciphers
"""
# ****************************************************************************
#       Copyright (C) 2007 David Kohel <kohel@maths.usyd.edu.au>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.misc.lazy_import import lazy_import
from sage.monoids.string_monoid_element import StringMonoidElement

lazy_import('sage.modules.free_module', 'FreeModule')

from .cipher import SymmetricKeyCipher


class AffineCipher(SymmetricKeyCipher):
    r"""
    Affine cipher class. This is the class that does the actual work of
    encryption and decryption. Users should not directly instantiate or
    create objects of this class. Instead, functionalities of this class
    should be accessed via
    :class:`AffineCryptosystem <sage.crypto.classical.AffineCryptosystem>`
    as the latter provides a convenient user interface.
    """

    def __init__(self, parent, key):
        r"""
        Create an affine cipher.

        INPUT:

        - ``parent`` -- an ``AffineCryptosystem`` object

        - ``key`` -- a secret key; let `N` be the size of the cipher domain.
          A key of this affine cipher is an ordered pair
          `(a, b) \in \ZZ_N \times \ZZ_N` such that `\gcd(a, N) = 1`.

        EXAMPLES:

        Testing of dumping and loading object::

            sage: A = AffineCryptosystem(AlphabeticStrings())
            sage: AC = A(3, 5)
            sage: AC == loads(dumps(AC))
            True
        """
        SymmetricKeyCipher.__init__(self, parent, key)

    def __eq__(self, other):
        r"""
        Comparing this ``AffineCipher`` with ``other``. Two ``AffineCipher``
        objects are the same if they are of the same type, have the same
        parent, and share the same secret key.

        INPUT:

        - ``other`` -- another object to compare with

        OUTPUT: ``True`` if ``self`` and ``other`` are the same
        ``AffineCipher`` object; ``False`` otherwise

        EXAMPLES::

            sage: aff1 = AffineCryptosystem(AlphabeticStrings())
            sage: aff2 = AffineCryptosystem(AlphabeticStrings())
            sage: aff1 == aff2
            True
            sage: aff1(1, 2) == aff2(1, 2)
            True
        """
        return type(self) is type(other) and self.parent() == other.parent() and self.key() == other.key()

    def __call__(self, M):
        r"""
        Return the ciphertext (respectively, plaintext) corresponding to
        ``M``. This is the main place where encryption and decryption takes
        place.

        INPUT:

        - ``M`` -- a message to be encrypted or decrypted. This message must
          be encoded using the plaintext or ciphertext alphabet. The current
          behaviour is that the plaintext and ciphertext alphabets are the
          same alphabet.

        - ``algorithm`` -- (default: ``'encrypt'``) whether to use the
          encryption or decryption algorithm on ``M``. The flag ``'encrypt'``
          signifies using the encryption algorithm, while ``'decrypt'``
          signifies using the decryption algorithm. The only acceptable
          values for ``algorithm`` are: ``'encrypt'`` and ``'decrypt'``.

        OUTPUT: the ciphertext or plaintext corresponding to ``M``

        EXAMPLES::

            sage: A = AffineCryptosystem(AlphabeticStrings()); A
            Affine cryptosystem on Free alphabetic string monoid on A-Z
            sage: P = A.encoding("The affine cryptosystem generalizes the shift cipher.")
            sage: P
            THEAFFINECRYPTOSYSTEMGENERALIZESTHESHIFTCIPHER
            sage: a, b = (9, 13)
            sage: E = A(a, b); E
            Affine cipher on Free alphabetic string monoid on A-Z
            sage: C = E(P); C
            CYXNGGHAXFKVSCJTVTCXRPXAXKNIHEXTCYXTYHGCFHSYXK
            sage: aInv, bInv = A.inverse_key(a, b)
            sage: D = A(aInv, bInv); D
            Affine cipher on Free alphabetic string monoid on A-Z
            sage: D(C)
            THEAFFINECRYPTOSYSTEMGENERALIZESTHESHIFTCIPHER
            sage: D(C) == P
            True
            sage: D(C) == P == D(E(P))
            True
        """
        # sanity check
        D = self.domain()  # = plaintext_space = ciphertext_space
        if M.parent() != D:
            raise TypeError("Argument M must be a string in the plaintext/ciphertext space.")

        from sage.rings.finite_rings.integer_mod import Mod
        A = list(D.alphabet())     # plaintext/ciphertext alphabet as a list
        N = self.domain().ngens()  # number of elements in this alphabet
        a, b = self.key()          # encryption/decryption key (a,b)
        # Let I be the index list of M. That is, the i-th element of M has
        # index k in the cipher domain D. We store this cipher domain index
        # as the i-th element of I.
        I = [A.index(str(e)) for e in M]

        # Encrypt the plaintext M. For each element i in I, the ciphertext
        # corresponding to i is ai + b (mod N). This can also be used for
        # decryption, in which case (a, b) is the inverse key corresponding
        # to a secret key.
        return D([ A.index(A[Mod(a*i + b, N).lift()]) for i in I ])

    def _repr_(self):
        r"""
        Return the string representation of this affine cipher.

        EXAMPLES::

            sage: A = AffineCryptosystem(AlphabeticStrings())
            sage: A(1, 2)
            Affine cipher on Free alphabetic string monoid on A-Z
        """
        # The affine cipher has the plaintext and ciphertext spaces defined
        # over the same non-empty alphabet. The cipher domain is the same
        # as the alphabet used for the plaintext and ciphertext spaces.
        return "Affine cipher on %s" % self.parent().cipher_domain()


class HillCipher(SymmetricKeyCipher):
    """
    Hill cipher class
    """
    def __init__(self, parent, key):
        """
        Create a Hill cipher.

        EXAMPLES::

            sage: # needs sage.modules
            sage: S = AlphabeticStrings()
            sage: E = HillCryptosystem(S,3); E
            Hill cryptosystem on Free alphabetic string monoid on A-Z of block length 3
            sage: M = E.key_space()
            sage: A = M([[1,0,1],[0,1,1],[2,2,3]]); A
            [1 0 1]
            [0 1 1]
            [2 2 3]
            sage: e = E(A); e
            Hill cipher on Free alphabetic string monoid on A-Z of block length 3
            sage: e(S("LAMAISONBLANCHE"))
            JYVKSKQPELAYKPV

        TESTS::

            sage: S = AlphabeticStrings()
            sage: E = HillCryptosystem(S,3)                                             # needs sage.modules
            sage: E == loads(dumps(E))                                                  # needs sage.modules
            True
        """
        # TODO: some type checking that the key is an invertible matrix?
        SymmetricKeyCipher.__init__(self, parent, key)

    def __eq__(self, right):
        return type(self) is type(right) and self.parent() == right.parent() and self.key() == right.key()

    def __call__(self, M):
        S = self.domain() # = plaintext_space = ciphertext_space
        if not isinstance(M, StringMonoidElement) and M.parent() == S:
            raise TypeError("Argument M (= %s) must be a string in the plaintext space." % M)
        m = self.parent().block_length()
        if len(M) % m != 0:
            raise TypeError("The length of M (= %s) must be a multiple of %s." % (M, m ))
        Alph = list(S.alphabet())
        A = self.key() # A is an m x m matrix
        R = A.parent().base_ring()
        V = FreeModule(R,m)
        Mstr = str(M)
        C = []
        for i in range(len(M)//m):
            v = V([ Alph.index(Mstr[m*i+j]) for j in range(m) ])
            C += (v * A).list()
        return S([ k.lift() for k in C ])

    def _repr_(self):
        r"""
        Return the string representation of this Hill cipher.

        EXAMPLES::

            sage: # needs sage.modules
            sage: H = HillCryptosystem(AlphabeticStrings(), 3)
            sage: M = MatrixSpace(IntegerModRing(26), 3, 3)
            sage: A = M([[1,0,1], [0,1,1], [2,2,3]])
            sage: e = H(A); e
            Hill cipher on Free alphabetic string monoid on A-Z of block length 3
        """
        return "Hill cipher on %s of block length %s" % (
            self.parent().cipher_domain(), self.parent().block_length() )

    def inverse(self):
        E = self.parent()
        try:
            B = E.inverse_key(self.key())
        except Exception:
            raise ValueError("Argument\n\n%s\n\nmust be an invertible cipher." % self)
        return E(B)


class ShiftCipher(SymmetricKeyCipher):
    r"""
    Shift cipher class. This is the class that does the actual work of
    encryption and decryption. Users should not directly instantiate or
    create objects of this class. Instead, functionalities of this class
    should be accessed via
    :class:`ShiftCryptosystem <sage.crypto.classical.ShiftCryptosystem>`
    as the latter provides a convenient user interface.
    """

    def __init__(self, parent, key):
        r"""
        Create a shift cipher.

        INPUT:

        - ``parent`` -- a ``ShiftCryptosystem`` object

        - ``key`` -- a secret key

        EXAMPLES::

            sage: S = ShiftCryptosystem(AlphabeticStrings()); S
            Shift cryptosystem on Free alphabetic string monoid on A-Z
            sage: P = S.encoding("The shift cryptosystem generalizes the Caesar cipher.")
            sage: P
            THESHIFTCRYPTOSYSTEMGENERALIZESTHECAESARCIPHER
            sage: K = 7
            sage: C = S.enciphering(K, P); C
            AOLZOPMAJYFWAVZFZALTNLULYHSPGLZAOLJHLZHYJPWOLY
            sage: S.deciphering(K, C)
            THESHIFTCRYPTOSYSTEMGENERALIZESTHECAESARCIPHER
            sage: S.deciphering(K, C) == P
            True
        """
        SymmetricKeyCipher.__init__(self, parent, key)

    def __eq__(self, other):
        r"""
        Comparing this ``ShiftCipher`` with ``other``. Two ``ShiftCipher``
        objects are the same if they are of the same type, have the same
        parent, and share the same secret key.

        INPUT:

        - ``other`` -- another object to compare with

        OUTPUT: ``True`` if ``self`` and ``other`` are the same ``ShiftCipher``
        object; ``False`` otherwise.

        EXAMPLES::

            sage: shift1 = ShiftCryptosystem(AlphabeticStrings())
            sage: shift2 = ShiftCryptosystem(AlphabeticStrings())
            sage: shift1 == shift2
            True
            sage: shift1 = ShiftCryptosystem(HexadecimalStrings())
            sage: shift2 = ShiftCryptosystem(BinaryStrings())
            sage: shift1 == shift2
            False
        """
        return type(self) is type(other) and self.parent() == other.parent() and self.key() == other.key()

    def __call__(self, M):
        r"""
        Return the ciphertext (respectively, plaintext) corresponding to
        ``M``. This is the main place where encryption and decryption takes
        place.

        INPUT:

        - ``M`` -- a message to be encrypted or decrypted. This message must
          be encoded using the plaintext or ciphertext alphabet. The current
          behaviour is that the plaintext and ciphertext alphabets are the
          same alphabet.

        OUTPUT: the ciphertext or plaintext corresponding to ``M``

        EXAMPLES:

        These are indirect doctests. Functionalities of this class are
        usually invoked from
        :class:`ShiftCryptosystem <sage.crypto.classical.ShiftCryptosystem>`.

        ::

            sage: S = ShiftCryptosystem(AlphabeticStrings())
            sage: S.enciphering(12, S.encoding("Stop shifting me."))
            EFABETURFUZSYQ
            sage: S = ShiftCryptosystem(HexadecimalStrings())
            sage: S.enciphering(7, S.encoding("Shift me now."))
            cadfd0ddeb97d4dc97d5d6ee95
            sage: S = ShiftCryptosystem(BinaryStrings())
            sage: S.enciphering(1, S.encoding("OK, enough shifting."))
            1011000010110100110100111101111110011010100100011001000010001010100110001001011111011111100011001001011110010110100110011000101110010110100100011001100011010001
        """
        dom = self.domain()  # = plaintext_space = ciphertext_space
        if not isinstance(M, StringMonoidElement) and M.parent() == dom:
            raise TypeError("Argument M (= %s) must be a string in the plaintext/ciphertext space." % M)
        from sage.rings.finite_rings.integer_mod import Mod
        A = list(dom.alphabet())   # plaintext/ciphertext alphabet as a list
        N = self.domain().ngens()  # number of elements in this alphabet
        K = self.key()             # encryption/decryption key
        # Here, M is a message encoded within the ciphertext/plaintext
        # alphabet of this shift cryptosystem. The list A above is a list of
        # all elements of this alphabet, each element being associated with
        # an index 0 <= i < n, where n is the size of A. Now get the index
        # of each character in the message M, storing all these indices
        # in the index list I.
        # TODO: the following two lines are coded with clarity and
        # readability in mind. It is possible to remove the overhead of
        # doing a list comprehension to get the index associated with each
        # element of M. For example, the next two lines can be crammed into
        # one list comprehension as follows:
        # return dom([ A.index(A[Mod(A.index(str(i)) + K, N).lift()]) for i in M ])
        # But it performs badly compared to the following two lines.
        I = [A.index(str(e)) for e in M]
        # Perform encryption/decryption on the whole message M, returning
        # the result as a string encoded in the alphabet A.
        return dom([ A.index(A[Mod(i + K, N).lift()]) for i in I ])

    def _repr_(self):
        r"""
        Return the string representation of this shift cipher.

        EXAMPLES::

            sage: S = ShiftCryptosystem(AlphabeticStrings())
            sage: S(3)
            Shift cipher on Free alphabetic string monoid on A-Z
            sage: S = ShiftCryptosystem(HexadecimalStrings())
            sage: S(5)
            Shift cipher on Free hexadecimal string monoid
            sage: S = ShiftCryptosystem(BinaryStrings())
            sage: S(1)
            Shift cipher on Free binary string monoid
        """
        # The shift cipher has the plaintext and ciphertext spaces defined
        # over the same non-empty alphabet. The cipher domain is the same
        # as the alphabet used for the plaintext and ciphertext spaces.
        return "Shift cipher on %s" % self.parent().cipher_domain()


class SubstitutionCipher(SymmetricKeyCipher):
    """
    Substitution cipher class
    """
    def __init__(self, parent, key):
        """
        Create a substitution cipher.

        EXAMPLES::

            sage: S = AlphabeticStrings()
            sage: E = SubstitutionCryptosystem(S)
            sage: E
            Substitution cryptosystem on Free alphabetic string monoid on A-Z
            sage: K = S([ 25-i for i in range(26) ])
            sage: K
            ZYXWVUTSRQPONMLKJIHGFEDCBA
            sage: e = E(K)
            sage: m = S("THECATINTHEHAT")
            sage: e(m)
            GSVXZGRMGSVSZG

        TESTS::

            sage: S = AlphabeticStrings()
            sage: E = SubstitutionCryptosystem(S)
            sage: E == loads(dumps(E))
            True
        """
        SymmetricKeyCipher.__init__(self, parent, key)

    def __eq__(self, right):
        return type(self) is type(right) and self.parent() == right.parent() and self.key() == right.key()

    def __call__(self, M):
        S = self.domain() # = plaintext_space = ciphertext_space
        if not isinstance(M, StringMonoidElement) and M.parent() == S:
            raise TypeError("Argument M (= %s) must be a string in the plaintext space." % M)
        A = list(S.alphabet())
        K = str(self.key()) # K is a string, while we want the indices:
        I = [ A.index(K[i]) for i in range(len(K)) ]
        Mstr = str(M)
        return S([ I[A.index(Mstr[i])] for i in range(len(Mstr)) ])

    def _repr_(self):
        r"""
        Return the string representation of this substitution cipher.

        EXAMPLES::

            sage: A = HexadecimalStrings(); S = SubstitutionCryptosystem(A)
            sage: K = A([16-i for i in range(16)]); S(K)
            Substitution cipher on Free hexadecimal string monoid
            sage: A = AlphabeticStrings(); S = SubstitutionCryptosystem(A)
            sage: K = A([26-i for i in range(26)]); S(K)
            Substitution cipher on Free alphabetic string monoid on A-Z
            sage: A = OctalStrings(); S = SubstitutionCryptosystem(A)
            sage: K = A([8-i for i in range(8)]); S(K)
            Substitution cipher on Free octal string monoid
            sage: A = BinaryStrings(); S = SubstitutionCryptosystem(A)
            sage: K = A([2-i for i in range(2)]); S(K)
            Substitution cipher on Free binary string monoid
            sage: A = Radix64Strings(); S = SubstitutionCryptosystem(A)
            sage: K = A([64-i for i in range(64)]); S(K)
            Substitution cipher on Free radix 64 string monoid
        """
        return "Substitution cipher on %s" % self.parent().cipher_domain()

    def inverse(self):
        E = self.parent()
        K = E.inverse_key(self.key())
        return E(K)


class TranspositionCipher(SymmetricKeyCipher):
    """
    Transition cipher class
    """
    def __init__(self, parent, key):
        """
        Create a transposition cipher.

        EXAMPLES::

            sage: # needs sage.groups
            sage: S = AlphabeticStrings()
            sage: E = TranspositionCryptosystem(S,14); E
            Transposition cryptosystem on
             Free alphabetic string monoid on A-Z of block length 14
            sage: K = [ 14-i for i in range(14) ]; K
            [14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1]
            sage: e = E(K)
            sage: m = S("THECATINTHEHAT")
            sage: e(m)
            TAHEHTNITACEHT

        EXAMPLES::

            sage: # needs sage.groups
            sage: S = AlphabeticStrings()
            sage: E = TranspositionCryptosystem(S,15)
            sage: m = S("THECATANDTHEHAT")
            sage: G = E.key_space(); G
            Symmetric group of order 15! as a permutation group
            sage: g = G([ 3, 2, 1, 6, 5, 4, 9, 8, 7, 12, 11, 10, 15, 14, 13 ])
            sage: e = E(g)
            sage: e(m)
            EHTTACDNAEHTTAH

        TESTS::

            sage: S = AlphabeticStrings()
            sage: E = TranspositionCryptosystem(S,14)                                   # needs sage.groups
            sage: E == loads(dumps(E))                                                  # needs sage.groups
            True
        """
        n = parent.block_length()
        if isinstance(key, list) and not len(key) == n:
            raise ValueError("key (= %s) must have block length %s" % (key, n))
        SymmetricKeyCipher.__init__(self, parent, key)

    def __call__(self, M, mode='ECB'):
        S = self.domain() # = plaintext_space = ciphertext_space
        if not isinstance(M, StringMonoidElement) and M.parent() == S:
            raise TypeError("Argument M (= %s) must be a string in the plaintext space." % M)
        if not mode == "ECB":
            raise NotImplementedError("Enciphering not implemented for mode (= %s) other than 'ECB'." % mode)
        g = self.key()
        N = len(M)
        m = self.parent().block_length()
        if not N % m == 0:
            raise TypeError("Argument M (= %s) must be a string of length k*%s." % (M, m))
        Melt = M._element_list # this uses the internal structure of string monoids
        # Caution: this is parsed as an outer loop in k and an inner loop in i:
        #     for k in range(N//m):
        #         for i in range(m):
        #             S([ Melt[g(i+1)-1+k*m]
        return S([ Melt[g(i+1)-1+k*m] for k in range(N//m) for i in range(m) ])

    def inverse(self):
        E = self.parent()
        K = E.inverse_key(self.key())
        return E(K)


class VigenereCipher(SymmetricKeyCipher):
    """
    Vigenere cipher class
    """
    def __init__(self, parent, key):
        """
        Create a Vigenere cipher.

        EXAMPLES::

            sage: S = AlphabeticStrings()
            sage: E = VigenereCryptosystem(S,11)
            sage: K = S("SHAKESPEARE")
            sage: e = E(K)
            sage: m = S("THECATINTHEHAT")
            sage: e(m)
            LOEMELXRTYIZHT

        TESTS::

            sage: S = AlphabeticStrings()
            sage: E = VigenereCryptosystem(S,11)
            sage: E == loads(dumps(E))
            True
        """
        SymmetricKeyCipher.__init__(self, parent, key)

    def __call__(self, M, mode='ECB'):
        S = self.domain() # = plaintext_space = ciphertext_space
        if not isinstance(M, StringMonoidElement) and M.parent() == S:
            raise TypeError("Argument M (= %s) must be a string in the plaintext space." % M)
        if not mode == "ECB":
            raise NotImplementedError("Enciphering not implemented for mode (= %s) other than 'ECB'." % mode)
        K = self.key()
        m = self.parent().period()
        n = S.ngens()
        # This uses the internal structure of string monoids
        Melt = M._element_list
        Kelt = K._element_list
        return S([ (Melt[i]+Kelt[i % m]) % n for i in range(len(M)) ])

    def inverse(self):
        E = self.parent()
        K = E.inverse_key(self.key())
        return E(K)



"""
Octonions
"""

from sage.structure.element cimport AlgebraElement
from sage.modules.free_module_element cimport FreeModuleElement

cdef class Octonion_generic(AlgebraElement):
    cdef FreeModuleElement vec

    cpdef Octonion_generic conjugate(self)
    cpdef quadratic_form(self)
    cpdef norm(self)
    cpdef abs(self)
    cpdef real_part(self)
    cpdef Octonion_generic imag_part(self)

cdef class Octonion(Octonion_generic):
    pass



r"""
Convert PARI objects to Sage types
"""

# ****************************************************************************
#       Copyright (C) 2016 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#       Copyright (C) 2016 Luca De Feo <luca.defeo@polytechnique.edu>
#       Copyright (C) 2016 Vincent Delecroix <vincent.delecroix@u-bordeaux.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from cysignals.signals cimport sig_on, sig_off

from cypari2.types cimport (GEN, typ, t_INT, t_FRAC, t_REAL, t_COMPLEX,
                            t_INTMOD, t_PADIC, t_INFINITY, t_VEC, t_COL,
                            t_VECSMALL, t_MAT, t_STR,
                            lg, precp)
from cypari2.paridecl cimport (
    avma,
    divisorsu,
    FF_to_FpXQ_i,
    inf_get_sign,
    is_rational_t,
    maxprime,
    pari_PRIMES, 
    pari_sp, 
    t_FFELT,
    t_POLMOD,
    uisprime,
    uisprimepower,
    ulong, 
)
from cypari2.stack cimport new_gen
from sage.libs.pari.convert_gmp cimport INT_to_mpz, new_gen_from_mpz_t, new_gen_from_mpq_t, INTFRAC_to_mpq

from sage.ext.stdsage cimport PY_NEW
from sage.libs.gmp.mpz cimport mpz_fits_slong_p, mpz_sgn, mpz_get_ui, mpz_set, mpz_set_si, mpz_set_ui
from sage.libs.gmp.mpq cimport mpq_denref, mpq_numref
from sage.misc.lazy_import import LazyImport
from sage.rings.integer cimport smallInteger
from sage.rings.infinity import Infinity

try:
    from sage.rings.real_mpfr import RealField
    from sage.rings.complex_mpfr import ComplexField
except ImportError:
    pass

try:
    from sage.rings.number_field.number_field import QuadraticField
except ImportError:
    pass
else:
    QQi = QuadraticField(-1, 'i')

try:
    from sage.rings.padics.factory import Qp
except ImportError:
    pass

pari_typ_to_entries_type = LazyImport('sage.libs.pari.convert_sage_matrix', 'pari_typ_to_entries_type')


cpdef gen_to_sage(Gen z, locals=None):
    """
    Convert a PARI gen to a Sage/Python object.

    INPUT:

    - ``z`` -- PARI ``gen``

    - ``locals`` -- (optional) dictionary used in fallback cases that
      involve :func:`sage_eval`

    OUTPUT: one of the following depending on the PARI type of ``z``

    - a :class:`~sage.rings.integer.Integer` if ``z`` is an integer (type ``t_INT``)

    - a :class:`~sage.rings.rational.Rational` if ``z`` is a rational (type ``t_FRAC``)

    - a :class:`~sage.rings.real_mpfr.RealNumber` if ``z`` is a real
      number (type ``t_REAL``). The precision will be equivalent.

    - a :class:`~sage.rings.number_field.number_field_element_quadratic.NumberFieldElement_quadratic`
      or a :class:`~sage.rings.complex_mpfr.ComplexNumber` if ``z`` is a complex
      number (type ``t_COMPLEX``). The former is used when the real and imaginary parts are
      integers or rationals and the latter when they are floating point numbers. In that
      case The precision will be the maximal precision of the real and imaginary parts.

    - a Python list if ``z`` is a vector or a list (type ``t_VEC``, ``t_COL``)

    - a Python string if ``z`` is a string (type ``t_STR``)

    - a Python list of Python integers if ``z`` is a small vector (type ``t_VECSMALL``)

    - a matrix if ``z`` is a matrix (type ``t_MAT``)

    - a `p`-adic element (type ``t_PADIC``)

    - a :class:`~sage.rings.infinity.Infinity` if ``z`` is an infinity
      (type ``t_INF``)

    EXAMPLES::

        sage: from sage.libs.pari.convert_sage import gen_to_sage

    Converting an integer::

        sage: z = pari('12'); z
        12
        sage: z.type()
        't_INT'
        sage: a = gen_to_sage(z); a
        12
        sage: a.parent()
        Integer Ring

        sage: gen_to_sage(pari('7^42'))
        311973482284542371301330321821976049

    Converting a rational number::

        sage: z = pari('389/17'); z
        389/17
        sage: z.type()
        't_FRAC'
        sage: a = gen_to_sage(z); a
        389/17
        sage: a.parent()
        Rational Field

        sage: gen_to_sage(pari('5^30 / 3^50'))
        931322574615478515625/717897987691852588770249

    Converting a real number::

        sage: pari.set_real_precision(70)
        15
        sage: z = pari('1.234'); z
        1.234000000000000000000000000000000000000000000000000000000000000000000
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.real_mpfr
        1.234000000000000000000000000000000000000000000000000000000000000000000000000
        sage: a.parent()                                                                # needs sage.rings.real_mpfr
        Real Field with 256 bits of precision
        sage: pari.set_real_precision(15)
        70
        sage: a = gen_to_sage(pari('1.234')); a                                         # needs sage.rings.real_mpfr
        1.23400000000000000
        sage: a.parent()                                                                # needs sage.rings.real_mpfr
        Real Field with 64 bits of precision

    For complex numbers, the parent depends on the PARI type::

        sage: z = pari('(3+I)'); z
        3 + I
        sage: z.type()
        't_COMPLEX'
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.number_field
        i + 3
        sage: a.parent()                                                                # needs sage.rings.number_field
        Number Field in i with defining polynomial x^2 + 1 with i = 1*I

        sage: z = pari('(3+I)/2'); z
        3/2 + 1/2*I
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.number_field
        1/2*i + 3/2
        sage: a.parent()                                                                # needs sage.rings.number_field
        Number Field in i with defining polynomial x^2 + 1 with i = 1*I

        sage: z = pari('1.0 + 2.0*I'); z
        1.00000000000000 + 2.00000000000000*I
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.real_mpfr
        1.00000000000000000 + 2.00000000000000000*I
        sage: a.parent()                                                                # needs sage.rings.real_mpfr
        Complex Field with 64 bits of precision

        sage: z = pari('1 + 1.0*I'); z
        1 + 1.00000000000000*I
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.real_mpfr
        1.00000000000000000 + 1.00000000000000000*I
        sage: a.parent()                                                                # needs sage.rings.real_mpfr
        Complex Field with 64 bits of precision

        sage: z = pari('1.0 + 1*I'); z
        1.00000000000000 + I
        sage: a = gen_to_sage(z); a                                                     # needs sage.rings.real_mpfr
        1.00000000000000000 + 1.00000000000000000*I
        sage: a.parent()                                                                # needs sage.rings.real_mpfr
        Complex Field with 64 bits of precision

    Converting polynomials::

        sage: f = pari('(2/3)*x^3 + x - 5/7 + y')
        sage: f.type()
        't_POL'

        sage: R.<x,y> = QQ[]
        sage: gen_to_sage(f, {'x': x, 'y': y})
        2/3*x^3 + x + y - 5/7
        sage: parent(gen_to_sage(f, {'x': x, 'y': y}))
        Multivariate Polynomial Ring in x, y over Rational Field

        sage: # needs sage.symbolic
        sage: x,y = SR.var('x,y')
        sage: gen_to_sage(f, {'x': x, 'y': y})
        2/3*x^3 + x + y - 5/7
        sage: parent(gen_to_sage(f, {'x': x, 'y': y}))
        Symbolic Ring

        sage: gen_to_sage(f)
        Traceback (most recent call last):
        ...
        NameError: name 'x' is not defined

    Converting vectors::

        sage: # needs sage.rings.number_field sage.rings.real_mpfr
        sage: z1 = pari('[-3, 2.1, 1+I]'); z1
        [-3, 2.10000000000000, 1 + I]
        sage: z2 = pari('[1.0*I, [1,2]]~'); z2
        [1.00000000000000*I, [1, 2]]~
        sage: z1.type(), z2.type()
        ('t_VEC', 't_COL')
        sage: a1 = gen_to_sage(z1)
        sage: a2 = gen_to_sage(z2)
        sage: type(a1), type(a2)
        (<... 'list'>, <... 'list'>)
        sage: [parent(b) for b in a1]
        [Integer Ring,
         Real Field with 64 bits of precision,
         Number Field in i with defining polynomial x^2 + 1 with i = 1*I]
        sage: [parent(b) for b in a2]
        [Complex Field with 64 bits of precision, <... 'list'>]

        sage: z = pari('Vecsmall([1,2,3,4])')
        sage: z.type()
        't_VECSMALL'
        sage: a = gen_to_sage(z); a
        [1, 2, 3, 4]
        sage: type(a)
        <... 'list'>
        sage: [parent(b) for b in a]
        [<... 'int'>, <... 'int'>, <... 'int'>, <... 'int'>]

    Matrices::

        sage: z = pari('[1,2;3,4]')
        sage: z.type()
        't_MAT'

        sage: # needs sage.modules
        sage: a = gen_to_sage(z); a
        [1 2]
        [3 4]
        sage: a.parent()
        Full MatrixSpace of 2 by 2 dense matrices over Integer Ring

    Conversion of `p`-adics::

        sage: # needs sage.rings.padics
        sage: z = pari('569 + O(7^8)'); z
        2 + 4*7 + 4*7^2 + 7^3 + O(7^8)
        sage: a = gen_to_sage(z); a
        2 + 4*7 + 4*7^2 + 7^3 + O(7^8)
        sage: a.parent()
        7-adic Field with capped relative precision 8

    Conversion of infinities::

        sage: gen_to_sage(pari('oo'))
        +Infinity
        sage: gen_to_sage(pari('-oo'))
        -Infinity

    Conversion of strings::

        sage: s = pari('"foo"').sage(); s
        'foo'
        sage: type(s)
        <class 'str'>
    """
    cdef GEN g = z.g
    cdef long t = typ(g)
    cdef long tx, ty
    cdef Gen real, imag, prec, xprec, yprec

    if t == t_INT:
        return Integer(z)
    elif t == t_FRAC:
        return Rational(z)
    elif t == t_REAL:
        prec = z.bitprecision()
        if typ(prec.g) == t_INFINITY:
            sage_prec = 53
        else:
            sage_prec = prec
        return RealField(sage_prec)(z)
    elif t == t_COMPLEX:
        real = z.real()
        imag = z.imag()
        tx = typ(real.g)
        ty = typ(imag.g)
        if tx in [t_INTMOD, t_PADIC] or ty in [t_INTMOD, t_PADIC]:
            raise NotImplementedError("No conversion to python available for t_COMPLEX with t_INTMOD or t_PADIC components")
        if tx == t_REAL or ty == t_REAL:
            xprec = real.bitprecision()  # will be infinite if exact
            yprec = imag.bitprecision()  # will be infinite if exact
            if typ(xprec.g) == t_INFINITY:
                if typ(yprec.g) == t_INFINITY:
                    sage_prec = 53
                else:
                    sage_prec = yprec
            elif typ(yprec.g) == t_INFINITY:
                sage_prec = xprec
            else:
                sage_prec = max(xprec, yprec)

            R = RealField(sage_prec)
            C = ComplexField(sage_prec)
            return C(R(real), R(imag))
        else:
            return QQi([gen_to_sage(real), gen_to_sage(imag)])
    elif t == t_VEC or t == t_COL:
        return [gen_to_sage(x, locals) for x in z.python_list()]
    elif t == t_VECSMALL:
        return z.python_list_small()
    elif t == t_MAT:
        from .convert_sage_matrix import gen_to_sage_matrix
        return gen_to_sage_matrix(z, locals)
    elif t == t_PADIC:
        p = z.padicprime()
        K = Qp(Integer(p), precp(g))
        return K(z.lift())
    elif t == t_STR:
        return str(z)
    elif t == t_INFINITY:
        if inf_get_sign(g) >= 0:
            return Infinity
        else:
            return -Infinity

    # Fallback (e.g. polynomials): use string representation
    from sage.misc.sage_eval import sage_eval
    locals = {} if locals is None else locals
    return sage_eval(str(z), locals=locals)


cpdef set_integer_from_gen(Integer self, Gen x):
    r"""
    EXAMPLES::

        sage: [Integer(pari(x)) for x in [1, 2^60, 2., GF(3)(1), GF(9,'a')(2)]]         # needs sage.rings.finite_rings
        [1, 1152921504606846976, 2, 1, 2]
        sage: Integer(pari(2.1)) # indirect doctest
        Traceback (most recent call last):
        ...
        TypeError: Attempt to coerce non-integral real number to an Integer
    """
    # Simplify and lift until we get an integer
    while typ((<Gen>x).g) != t_INT:
        x = x.simplify()
        paritype = typ((<Gen>x).g)
        if paritype == t_INT:
            break
        elif paritype == t_REAL:
            # Check that the fractional part is zero
            if not x.frac().gequal0():
                raise TypeError("Attempt to coerce non-integral real number to an Integer")
            # floor yields an integer
            x = x.floor()
            break
        elif paritype == t_PADIC:
            if x._valp() < 0:
                raise TypeError("cannot convert p-adic with negative valuation to an integer")
            # Lifting a PADIC yields an integer
            x = x.lift()
            break
        elif paritype == t_INTMOD:
            # Lifting an INTMOD yields an integer
            x = x.lift()
            break
        elif paritype == t_POLMOD:
            x = x.lift()
        elif paritype == t_FFELT:
            # x = (f modulo defining polynomial of finite field);
            # we extract f.
            sig_on()
            x = new_gen(FF_to_FpXQ_i((<Gen>x).g))
        else:
            raise TypeError("Unable to coerce PARI %s to an Integer" % x)

    # Now we have a true PARI integer, convert it to Sage
    INT_to_mpz(self.value, (<Gen>x).g)


cpdef Gen new_gen_from_integer(Integer self):
    """
    TESTS::

        sage: Rational(pari(2))  # indirect doctest
        2
        sage: Rational(pari(-1))
        -1
    """
    return new_gen_from_mpz_t(self.value)


cpdef set_rational_from_gen(Rational self, Gen x):
    r"""
    EXAMPLES::

        sage: [Rational(pari(x)) for x in [1, 1/2, 2^60, 2., GF(3)(1), GF(9,'a')(2)]]   # needs sage.rings.finite_rings
        [1, 1/2, 1152921504606846976, 2, 1, 2]
        sage: Rational(pari(2.1)) # indirect doctest
        Traceback (most recent call last):
        ...
        TypeError: Attempt to coerce non-integral real number to an Integer
    """
    x = x.simplify()
    if is_rational_t(typ((<Gen>x).g)):
        INTFRAC_to_mpq(self.value, (<Gen>x).g)
    else:
        a = Integer(x)
        mpz_set(mpq_numref(self.value), a.value)
        mpz_set_si(mpq_denref(self.value), 1)


cpdef Gen new_gen_from_rational(Rational self):
    """
    TESTS::

        sage: Integer(pari(2/2))  # indirect doctest
        1
        sage: Rational(pari(-1/2))
        -1/2
    """
    return new_gen_from_mpq_t(self.value)


cpdef list pari_divisors_small(Integer self):
    r"""
    Return the list of divisors of this number using PARI ``divisorsu``.

    .. SEEALSO::

        This method is better used through :meth:`sage.rings.integer.Integer.divisors`.

    EXAMPLES::

        sage: from sage.libs.pari.convert_sage import pari_divisors_small
        sage: pari_divisors_small(4)
        [1, 2, 4]

    The integer must fit into an unsigned long::

        sage: pari_divisors_small(-4)
        Traceback (most recent call last):
        ...
        AssertionError
        sage: pari_divisors_small(2**65)
        Traceback (most recent call last):
        ...
        AssertionError
    """
    # we need n to fit into a long and not a unsigned long in order to use
    # smallInteger
    assert mpz_fits_slong_p(self.value) and mpz_sgn(self.value) > 0

    cdef unsigned long n = mpz_get_ui(self.value)

    global avma
    cdef pari_sp ltop = avma
    cdef GEN d
    cdef list output

    try:
        sig_on()
        d = divisorsu(n)
        sig_off()
        output = [smallInteger(d[i]) for i in range(1,lg(d))]
        return output
    finally:
        avma = ltop


cpdef pari_is_prime(Integer p):
    r"""
    Return whether ``p`` is a prime.

    The caller must ensure that ``p.value`` fits in a long.

    EXAMPLES::

        sage: from sage.libs.pari.convert_sage import pari_is_prime
        sage: pari_is_prime(2)
        True
        sage: pari_is_prime(3)
        True
        sage: pari_is_prime(1)
        False
        sage: pari_is_prime(4)
        False

    Its recommended to use :meth:`sage.rings.integer.Integer.is_prime`, which checks overflow.
    The following is incorrect, because the number does not fit into a long::

        sage: pari_is_prime(2**64 + 2)
        True
    """
    return bool(uisprime(mpz_get_ui(p.value)))


cpdef pari_is_prime_power(Integer q, bint get_data):
    r"""
    Return whether ``q`` is a prime power.

    The caller must ensure that ``q.value`` fits in a long.

    OUTPUT:

    If ``get_data`` return a tuple of the prime and the exponent.
    Otherwise return a boolean.

    EXAMPLES::

        sage: from sage.libs.pari.convert_sage import pari_is_prime_power
        sage: pari_is_prime_power(2, False)
        True
        sage: pari_is_prime_power(2, True)
        (2, 1)
        sage: pari_is_prime_power(4, False)
        True
        sage: pari_is_prime_power(4, True)
        (2, 2)
        sage: pari_is_prime_power(6, False)
        False
        sage: pari_is_prime_power(6, True)
        (6, 0)

    Its recommended to use :meth:`sage.rings.integer.Integer.is_prime_power`, which checks overflow.
    The following is incorrect, because the number does not fit into a long::

        sage: pari_is_prime_power(2**64 + 2, False)
        True
    """
    cdef long p, n
    n = uisprimepower(mpz_get_ui(q.value), <ulong*>(&p))
    if n:
        return (smallInteger(p), smallInteger(n)) if get_data else True
    else:
        return (q, smallInteger(0)) if get_data else False


cpdef unsigned long pari_maxprime() noexcept:
    """
    Return to which limit PARI has computed the primes.

    EXAMPLES::

        sage: from sage.libs.pari.convert_sage import pari_maxprime
        sage: a = pari_maxprime()
        sage: res = prime_range(2, 2*a)
        sage: b = pari_maxprime()
        sage: b >= 2*a
        True
    """
    return maxprime()


cpdef list pari_prime_range(long c_start, long c_stop, bint py_ints=False):
    """
    Return a list of all primes between ``start`` and ``stop - 1``, inclusive.

    .. SEEALSO::

        :func:`sage.rings.fast_arith.prime_range`

    TESTS::

        sage: from sage.libs.pari.convert_sage import pari_prime_range
        sage: pari_prime_range(2, 19)
        [2, 3, 5, 7, 11, 13, 17]
    """
    cdef ulong i = 1
    res = []
    while pari_PRIMES[i] < c_start:
        i+=1
    while pari_PRIMES[i] < c_stop:
        if py_ints:
            res.append(pari_PRIMES[i])
        else:
            z = <Integer>PY_NEW(Integer)
            mpz_set_ui(z.value, pari_PRIMES[i])
            res.append(z)
        i+=1
    return res



# sage.doctest: needs sage.libs.pari
r"""
Generic spaces of modular forms

EXAMPLES (computation of base ring): Return the base ring of this
space of modular forms.

EXAMPLES: For spaces of modular forms for `\Gamma_0(N)` or
`\Gamma_1(N)`, the default base ring is
`\QQ`::

    sage: ModularForms(11,2).base_ring()
    Rational Field
    sage: ModularForms(1,12).base_ring()
    Rational Field
    sage: CuspForms(Gamma1(13),3).base_ring()
    Rational Field

The base ring can be explicitly specified in the constructor
function.

::

    sage: ModularForms(11,2,base_ring=GF(13)).base_ring()
    Finite Field of size 13

For modular forms with character the default base ring is the field
generated by the image of the character.

::

    sage: ModularForms(DirichletGroup(13).0,3).base_ring()
    Cyclotomic Field of order 12 and degree 4

For example, if the character is quadratic then the field is
`\QQ` (if the characteristic is `0`).

::

    sage: ModularForms(DirichletGroup(13).0^6,3).base_ring()
    Rational Field

An example in characteristic `7`::

    sage: ModularForms(13,3,base_ring=GF(7)).base_ring()
    Finite Field of size 7

AUTHORS:

- William Stein (2007): first version
"""

#########################################################################
#       Copyright (C) 2004--2006 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
#########################################################################

from sage.arith.misc import gcd
from sage.matrix.constructor import zero_matrix
from sage.misc.cachefunc import cached_method

import sage.modular.hecke.all as hecke
import sage.modular.arithgroup.all as arithgroup
from sage.modular import dirichlet

from sage.rings.infinity import PlusInfinity
from sage.rings.integer import Integer
from sage.rings.integer_ring import ZZ
from sage.rings.power_series_ring import PowerSeriesRing
from sage.rings.power_series_ring_element import PowerSeries
from sage.rings.rational_field import QQ
from sage.categories.rings import Rings

from sage.structure.all import Sequence
from sage.structure.richcmp import (richcmp_method, richcmp, rich_to_bool,
                                    richcmp_not_equal)

from .element import ModularFormElement, Newform
from . import defaults
from . import hecke_operator_on_qexp

WARN = False


@richcmp_method
class ModularFormsSpace(hecke.HeckeModule_generic):
    """
    A generic space of modular forms.
    """
    Element = ModularFormElement

    def __init__(self, group, weight, character, base_ring, category=None):
        r"""
        Generic spaces of modular forms. For spaces of modular forms for
        `\Gamma_0(N)` or `\Gamma_1(N)`, the default base
        ring is `\QQ`.

        EXAMPLES::

            sage: ModularForms(11,2)
            Modular Forms space of dimension 2 for Congruence Subgroup Gamma0(11) of weight 2 over Rational Field

        ::

            sage: ModularForms(11,2,base_ring=GF(13))
            Modular Forms space of dimension 2 for Congruence Subgroup Gamma0(11) of weight 2 over Finite Field of size 13

        ::

            sage: ModularForms(DirichletGroup(13).0,3)
            Modular Forms space of dimension 3, character [zeta12] and weight 3 over Cyclotomic Field of order 12 and degree 4

        ::

            sage: M = ModularForms(11,2)
            sage: M == loads(dumps(M))
            True
        """
        global WARN
        if WARN:
            print("Modular forms -- under development -- do not trust yet.")
            WARN = False
        if not isinstance(group, arithgroup.CongruenceSubgroupBase):
            raise TypeError("group (=%s) must be a congruence subgroup" % group)
        weight = Integer(weight)
        if not ((character is None) or isinstance(character, dirichlet.DirichletCharacter)):
            raise TypeError("character must be a Dirichlet character")
        if base_ring not in Rings():
            raise TypeError("base_ring must be a ring")
        self.__sturm_bound = None
        self.__weight, self.__group, self.__character = weight, group, character
        hecke.HeckeModule_generic.__init__(self, base_ring, group.level(), category=category)

    def prec(self, new_prec=None):
        """
        Return or set the default precision used for displaying
        `q`-expansions of elements of this space.

        INPUT:

        - ``new_prec`` -- positive integer (default: ``None``)

        OUTPUT: if new_prec is None, returns the current precision

        EXAMPLES::

            sage: M = ModularForms(1,12)
            sage: S = M.cuspidal_subspace()
            sage: S.prec()
            6
            sage: S.basis()
            [q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 + O(q^6)]
            sage: S.prec(8)
            8
            sage: S.basis()
            [q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 - 6048*q^6 - 16744*q^7 + O(q^8)]
        """
        return self.ambient().prec(new_prec)

    def set_precision(self, new_prec):
        """
        Set the default precision used for displaying
        `q`-expansions.

        INPUT:

        - ``new_prec`` -- positive integer

        EXAMPLES::

            sage: M = ModularForms(Gamma0(37),2)
            sage: M.set_precision(10)
            sage: S = M.cuspidal_subspace()
            sage: S.basis()
            [q + q^3 - 2*q^4 - q^7 - 2*q^9 + O(q^10),
             q^2 + 2*q^3 - 2*q^4 + q^5 - 3*q^6 - 4*q^9 + O(q^10)]

        ::

            sage: S.set_precision(0)
            sage: S.basis()
            [O(q^0), O(q^0)]

        The precision of subspaces is the same as the precision of the
        ambient space.

        ::

            sage: S.set_precision(2)
            sage: M.basis()
            [q + O(q^2), O(q^2), 1 + 2/3*q + O(q^2)]

        The precision must be nonnegative::

            sage: S.set_precision(-1)
            Traceback (most recent call last):
            ...
            ValueError: n (=-1) must be >= 0

        We do another example with nontrivial character.

        ::

            sage: M = ModularForms(DirichletGroup(13).0^2)
            sage: M.set_precision(10)
            sage: M.cuspidal_subspace().0
            q + (-zeta6 - 1)*q^2 + (2*zeta6 - 2)*q^3 + zeta6*q^4 + (-2*zeta6 + 1)*q^5 + (-2*zeta6 + 4)*q^6 + (2*zeta6 - 1)*q^8 - zeta6*q^9 + O(q^10)
        """
        self.ambient().set_precision(new_prec)

    def weight(self):
        """
        Return the weight of this space of modular forms.

        EXAMPLES::

            sage: M = ModularForms(Gamma1(13),11)
            sage: M.weight()
            11

        ::

            sage: M = ModularForms(Gamma0(997),100)
            sage: M.weight()
            100

        ::

            sage: M = ModularForms(Gamma0(97),4)
            sage: M.weight()
            4
            sage: M.eisenstein_submodule().weight()
            4
        """
        return self.__weight

    def group(self):
        r"""
        Return the congruence subgroup associated to this space of modular
        forms.

        EXAMPLES::

            sage: ModularForms(Gamma0(12),4).group()
            Congruence Subgroup Gamma0(12)

        ::

            sage: CuspForms(Gamma1(113),2).group()
            Congruence Subgroup Gamma1(113)

        Note that `\Gamma_1(1)` and `\Gamma_0(1)` are replaced by
        `\SL_2(\ZZ)`.

        ::

            sage: CuspForms(Gamma1(1),12).group()
            Modular Group SL(2,Z)
            sage: CuspForms(SL2Z,12).group()
            Modular Group SL(2,Z)
        """
        return self.__group

    def character(self):
        r"""
        Return the Dirichlet character corresponding to this space of
        modular forms. Returns None if there is no specific character
        corresponding to this space, e.g., if this is a space of modular
        forms on `\Gamma_1(N)` with `N>1`.

        EXAMPLES: The trivial character::

            sage: ModularForms(Gamma0(11),2).character()
            Dirichlet character modulo 11 of conductor 1 mapping 2 |--> 1

        Spaces of forms with nontrivial character::

            sage: ModularForms(DirichletGroup(20).0,3).character()
            Dirichlet character modulo 20 of conductor 4 mapping 11 |--> -1, 17 |--> 1

            sage: M = ModularForms(DirichletGroup(11).0, 3)
            sage: M.character()
            Dirichlet character modulo 11 of conductor 11 mapping 2 |--> zeta10
            sage: s = M.cuspidal_submodule()
            sage: s.character()
            Dirichlet character modulo 11 of conductor 11 mapping 2 |--> zeta10
            sage: CuspForms(DirichletGroup(11).0,3).character()
            Dirichlet character modulo 11 of conductor 11 mapping 2 |--> zeta10

        A space of forms with no particular character (hence None is
        returned)::

            sage: print(ModularForms(Gamma1(11),2).character())
            None

        If the level is one then the character is trivial.

        ::

            sage: ModularForms(Gamma1(1),12).character()
            Dirichlet character modulo 1 of conductor 1
        """
        return self.__character

    def has_character(self) -> bool:
        r"""
        Return ``True`` if this space of modular forms has a specific
        character.

        This is ``True`` exactly when the ``character()`` function does not
        return ``None``.

        EXAMPLES: A space for `\Gamma_0(N)` has trivial character,
        hence has a character.

        ::

            sage: CuspForms(Gamma0(11),2).has_character()
            True

        A space for `\Gamma_1(N)` (for `N\geq 2`) never
        has a specific character.

        ::

            sage: CuspForms(Gamma1(11),2).has_character()
            False
            sage: CuspForms(DirichletGroup(11).0,3).has_character()
            True
        """
        return self.character() is not None

    def is_ambient(self) -> bool:
        """
        Return ``True`` if this an ambient space of modular forms.

        EXAMPLES::

            sage: M = ModularForms(Gamma1(4),4)
            sage: M.is_ambient()
            True

        ::

            sage: E = M.eisenstein_subspace()
            sage: E.is_ambient()
            False
        """
        return False   # returning True is defined in the derived AmbientSpace class.

    def __normalize_prec(self, prec):
        """
        If prec=None, return self.prec(). Otherwise, make sure prec is a
        sensible choice of precision and return it.

        EXAMPLES::

            sage: N = ModularForms(6,4)
            sage: N._ModularFormsSpace__normalize_prec(int(3))
            3

        ::

            sage: type(N._ModularFormsSpace__normalize_prec(int(3)))
            <class 'sage.rings.integer.Integer'>
        """
        if prec is None:
            prec = self.prec()
        else:
            prec = Integer(prec)
        if prec < 0:
            raise ValueError("prec (=%s) must be at least 0" % prec)
        return prec

    @cached_method
    def echelon_form(self):
        r"""
        Return a space of modular forms isomorphic to ``self`` but with basis
        of `q`-expansions in reduced echelon form.

        This is useful, e.g., the default basis for spaces of modular forms
        is rarely in echelon form, but echelon form is useful for quickly
        recognizing whether a `q`-expansion is in the space.

        EXAMPLES: We first illustrate two ambient spaces and their echelon
        forms.

        ::

            sage: M = ModularForms(11)
            sage: M.basis()
            [q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6),
             1 + 12/5*q + 36/5*q^2 + 48/5*q^3 + 84/5*q^4 + 72/5*q^5 + O(q^6)]
            sage: M.echelon_form().basis()
            [1 + 12*q^2 + 12*q^3 + 12*q^4 + 12*q^5 + O(q^6),
             q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6)]

        ::

            sage: M = ModularForms(Gamma1(6),4)
            sage: M.basis()
            [q - 2*q^2 - 3*q^3 + 4*q^4 + 6*q^5 + O(q^6),
             1 + O(q^6),
             q - 8*q^4 + 126*q^5 + O(q^6),
             q^2 + 9*q^4 + O(q^6),
             q^3 + O(q^6)]
            sage: M.echelon_form().basis()
            [1 + O(q^6),
             q + 94*q^5 + O(q^6),
             q^2 + 36*q^5 + O(q^6),
             q^3 + O(q^6),
             q^4 - 4*q^5 + O(q^6)]

        We create a space with a funny basis then compute the corresponding
        echelon form.

        ::

            sage: M = ModularForms(11,4)
            sage: M.basis()
            [q + 3*q^3 - 6*q^4 - 7*q^5 + O(q^6),
             q^2 - 4*q^3 + 2*q^4 + 8*q^5 + O(q^6),
             1 + O(q^6),
             q + 9*q^2 + 28*q^3 + 73*q^4 + 126*q^5 + O(q^6)]
            sage: F = M.span_of_basis([M.0 + 1/3*M.1, M.2 + M.3]); F.basis()
            [q + 1/3*q^2 + 5/3*q^3 - 16/3*q^4 - 13/3*q^5 + O(q^6),
             1 + q + 9*q^2 + 28*q^3 + 73*q^4 + 126*q^5 + O(q^6)]
            sage: E = F.echelon_form(); E.basis()
            [1 + 26/3*q^2 + 79/3*q^3 + 235/3*q^4 + 391/3*q^5 + O(q^6),
             q + 1/3*q^2 + 5/3*q^3 - 16/3*q^4 - 13/3*q^5 + O(q^6)]
        """
        return self.span_of_basis(self.echelon_basis())

    @cached_method
    def echelon_basis(self):
        """
        Return a basis for ``self`` in reduced echelon form. This means that if
        we view the `q`-expansions of the basis as defining rows of
        a matrix (with infinitely many columns), then this matrix is in
        reduced echelon form.

        EXAMPLES::

            sage: M = ModularForms(Gamma0(11),4)
            sage: M.echelon_basis()
            [1 + O(q^6),
             q - 9*q^4 - 10*q^5 + O(q^6),
             q^2 + 6*q^4 + 12*q^5 + O(q^6),
             q^3 + q^4 + q^5 + O(q^6)]
            sage: M.cuspidal_subspace().echelon_basis()
            [q + 3*q^3 - 6*q^4 - 7*q^5 + O(q^6), q^2 - 4*q^3 + 2*q^4 + 8*q^5 + O(q^6)]

        ::

            sage: M = ModularForms(SL2Z, 12)
            sage: M.echelon_basis()
            [1 + 196560*q^2 + 16773120*q^3 + 398034000*q^4 + 4629381120*q^5 + O(q^6),
             q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 + O(q^6)]

        ::

            sage: M = CuspForms(Gamma0(17),4, prec=10)
            sage: M.echelon_basis()
            [q + 2*q^5 - 8*q^7 - 8*q^8 + 7*q^9 + O(q^10),
             q^2 - 3/2*q^5 - 7/2*q^6 + 9/2*q^7 + q^8 - 4*q^9 + O(q^10),
             q^3 - 2*q^6 + q^7 - 4*q^8 - 2*q^9 + O(q^10),
             q^4 - 1/2*q^5 - 5/2*q^6 + 3/2*q^7 + 2*q^9 + O(q^10)]
        """
        F = self.free_module()
        W = self._q_expansion_module()
        pr = W.degree()
        B = self.q_echelon_basis(pr)
        E = [self(F.linear_combination_of_basis(W.coordinates(f.padded_list(pr))))
             for f in B]
        return Sequence(E, cr=True, immutable=True)

    @cached_method
    def integral_basis(self):
        """
        Return an integral basis for this space of modular forms.

        EXAMPLES:

        In this example the integral and echelon bases are
        different. ::

            sage: m = ModularForms(97,2,prec=10)
            sage: s = m.cuspidal_subspace()
            sage: s.integral_basis()
            [q + 2*q^7 + 4*q^8 - 2*q^9 + O(q^10),
             q^2 + q^4 + q^7 + 3*q^8 - 3*q^9 + O(q^10),
             q^3 + q^4 - 3*q^8 + q^9 + O(q^10),
             2*q^4 - 2*q^8 + O(q^10),
             q^5 - 2*q^8 + 2*q^9 + O(q^10),
             q^6 + 2*q^7 + 5*q^8 - 5*q^9 + O(q^10),
             3*q^7 + 6*q^8 - 4*q^9 + O(q^10)]
            sage: s.echelon_basis()
            [q + 2/3*q^9 + O(q^10),
             q^2 + 2*q^8 - 5/3*q^9 + O(q^10),
             q^3 - 2*q^8 + q^9 + O(q^10),
             q^4 - q^8 + O(q^10),
             q^5 - 2*q^8 + 2*q^9 + O(q^10),
             q^6 + q^8 - 7/3*q^9 + O(q^10),
             q^7 + 2*q^8 - 4/3*q^9 + O(q^10)]

        Here's another example where there is a big gap in the valuations::

            sage: m = CuspForms(64,2)
            sage: m.integral_basis()
            [q + O(q^6), q^2 + O(q^6), q^5 + O(q^6)]

        TESTS::

            sage: m = CuspForms(11*2^4,2, prec=13); m
            Cuspidal subspace of dimension 19 of Modular Forms space of dimension 30 for Congruence Subgroup Gamma0(176) of weight 2 over Rational Field
            sage: m.integral_basis()          # takes a long time (3 or 4 seconds)
            [q + O(q^13),
             q^2 + O(q^13),
             q^3 + O(q^13),
             q^4 + O(q^13),
             q^5 + O(q^13),
             q^6 + O(q^13),
             q^7 + O(q^13),
             q^8 + O(q^13),
             q^9 + O(q^13),
             q^10 + O(q^13),
             q^11 + O(q^13),
             q^12 + O(q^13),
             O(q^13),
             O(q^13),
             O(q^13),
             O(q^13),
             O(q^13),
             O(q^13),
             O(q^13)]
        """
        W = self._q_expansion_module()
        pr = W.degree()
        B = self.q_integral_basis(pr)
        I = [self.linear_combination_of_basis(
            W.coordinates(f.padded_list(pr))) for f in B]
        return Sequence(I, cr=True, immutable=True)

    @cached_method
    def _q_expansion_module(self):
        """
        Return module spanned by coefficients of `q`-expansions to sufficient
        precision to determine elements of this space.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M._q_expansion_module()
            Vector space of degree 3 and dimension 2 over Rational Field
            User basis matrix:
            [   0    1   -2]
            [   1 12/5 36/5]
            sage: CuspForms(1,12)._q_expansion_module()
            Vector space of degree 2 and dimension 1 over Rational Field
            User basis matrix:
            [0 1]
        """
        prec = self.sturm_bound()
        C = self.q_expansion_basis(prec)
        V = self.base_ring()**prec
        return V.span_of_basis([f.padded_list(prec) for f in C])

    def q_expansion_basis(self, prec=None):
        """
        Return a sequence of `q`-expansions for the basis of this space
        computed to the given input precision.

        INPUT:

        - ``prec`` -- integer (>=0) or None

        If prec is None, the prec is computed to be *at least* large
        enough so that each `q`-expansion determines the form as an element
        of this space.

        .. NOTE::

           In fact, the `q`-expansion basis is always computed to
           *at least* ``self.prec()``.

        EXAMPLES::

            sage: S = ModularForms(11,2).cuspidal_submodule()
            sage: S.q_expansion_basis()
            [q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6)]
            sage: S.q_expansion_basis(5)
            [q - 2*q^2 - q^3 + 2*q^4 + O(q^5)]
            sage: S = ModularForms(1,24).cuspidal_submodule()
            sage: S.q_expansion_basis(8)
            [q + 195660*q^3 + 12080128*q^4 + 44656110*q^5 - 982499328*q^6 - 147247240*q^7 + O(q^8),
             q^2 - 48*q^3 + 1080*q^4 - 15040*q^5 + 143820*q^6 - 985824*q^7 + O(q^8)]

        An example which used to be buggy::

            sage: M = CuspForms(128, 2, prec=3)
            sage: M.q_expansion_basis()
            [q - q^17 + O(q^22),
             q^2 - 3*q^18 + O(q^22),
             q^3 - q^11 + q^19 + O(q^22),
             q^4 - 2*q^20 + O(q^22),
             q^5 - 3*q^21 + O(q^22),
             q^7 - q^15 + O(q^22),
             q^9 - q^17 + O(q^22),
             q^10 + O(q^22),
             q^13 - q^21 + O(q^22)]
        """
        if prec is None:
            try:  # don't care about precision -- just must be big enough to determine forms
                return self.__q_expansion_basis[1]
            except AttributeError:
                pass
            prec = -1  # big enough to determine forms
        else:
            prec = Integer(self.__normalize_prec(prec))

        if prec == 0:
            z = self._q_expansion_ring()(0, prec)
            return Sequence([z] * int(self.dimension()), immutable=True, cr=True)
        if prec != -1:
            try:
                current_prec, B = self.__q_expansion_basis
                if current_prec == prec:
                    return B
                if current_prec > prec:
                    return Sequence([f.add_bigoh(prec) for f in B], immutable=True, cr=True)
            except AttributeError:
                pass

        d = self.dimension()
        current_prec = max(prec, self.prec(), int(1.2 * d) + 3)  # +3 for luck.
        tries = 0
        while True:
            B = [f for f in self._compute_q_expansion_basis(current_prec) if f != 0]
            if len(B) == d:
                break
            tries += 1
            current_prec += d
            if tries > 5:
                print("WARNING: possible bug in q_expansion_basis for modular forms space %s" % self)
        if prec == -1:
            prec = current_prec
        B = Sequence(B, immutable=True, cr=True)
        self.__q_expansion_basis = (current_prec, B)
        if current_prec == prec:
            return B
        return Sequence([f.add_bigoh(prec) for f in B], immutable=True, cr=True)

    def _compute_q_expansion_basis(self, prec):
        """
        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: sage.modular.modform.space.ModularFormsSpace(Gamma0(11), 2, DirichletGroup(1)[0], QQ)._compute_q_expansion_basis(5)
            Traceback (most recent call last):
            ...
            NotImplementedError: this must be implemented in the derived class
        """
        raise NotImplementedError("this must be implemented in the derived class")

    def q_echelon_basis(self, prec=None):
        r"""
        Return the echelon form of the basis of `q`-expansions of
        ``self`` up to precision ``prec``.

        The `q`-expansions are power series (not actual modular
        forms). The number of `q`-expansions returned equals the
        dimension.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M.q_expansion_basis()
            [q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6),
             1 + 12/5*q + 36/5*q^2 + 48/5*q^3 + 84/5*q^4 + 72/5*q^5 + O(q^6)]

        ::

            sage: M.q_echelon_basis()
            [1 + 12*q^2 + 12*q^3 + 12*q^4 + 12*q^5 + O(q^6),
             q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6)]
        """
        prec = self.__normalize_prec(prec)
        if prec == 0:
            z = self._q_expansion_ring()(0, 0)
            return Sequence([z] * int(self.dimension()), cr=True)
        try:
            current_prec, B = self.__q_echelon_basis
        except AttributeError:
            current_prec, B = -1, []
        if current_prec == prec:
            return B
        if current_prec > prec:
            return Sequence([f.add_bigoh(prec) for f in B], cr=True)

        B = self.q_expansion_basis(prec)
        R = self.base_ring()
        A = R**prec
        gens = [f.padded_list(prec) for f in B]
        C = A.span(gens)

        T = self._q_expansion_ring()
        S = [T(f.list(), prec) for f in C.basis()]
        for _ in range(self.dimension() - len(S)):
            S.append(T(0, prec))
        S = Sequence(S, immutable=True, cr=True)
        self.__q_echelon_basis = (prec, S)
        return S

    def q_integral_basis(self, prec=None):
        r"""
        Return a `\ZZ`-reduced echelon basis of
        `q`-expansions for ``self``.

        The `q`-expansions are power series with coefficients in
        `\ZZ`; they are *not* actual modular forms.

        The base ring of ``self`` must be `\QQ`. The number of
        `q`-expansions returned equals the dimension.

        EXAMPLES::

            sage: S = CuspForms(11,2)
            sage: S.q_integral_basis(5)
            [q - 2*q^2 - q^3 + 2*q^4 + O(q^5)]
        """
        if not self.base_ring() == QQ:
            raise TypeError("the base ring must be Q")
        prec = self.__normalize_prec(prec)
        R = PowerSeriesRing(ZZ, name=defaults.DEFAULT_VARIABLE)
        if prec == 0:
            z = R(0, prec)
            return Sequence([z] * int(self.dimension()), cr=True)
        try:
            current_prec, B = self.__q_integral_basis
        except AttributeError:
            current_prec, B = -1, Sequence([], cr=True, immutable=True)

        if current_prec == prec:
            return B
        if current_prec > prec:
            return Sequence([f.add_bigoh(prec) for f in B], cr=True)

        B = self.q_expansion_basis(prec)

        # It's over Q; we just need to intersect it with ZZ^n.
        A = ZZ**prec
        gens = [f.padded_list(prec) for f in B]
        C = A.span(gens)
        D = C.saturation()
        S = [R(f.list(), prec) for f in D.basis()]
        for _ in range(self.dimension() - len(S)):
            S.append(R(0, prec))
        S = Sequence(S, immutable=True, cr=True)
        self.__q_integral_basis = (prec, S)
        return S

    @cached_method
    def _q_expansion_ring(self):
        """
        Return the parent for `q`-expansions of modular forms in ``self``.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M._q_expansion_ring()
            Power Series Ring in q over Rational Field
        """
        return PowerSeriesRing(self.base_ring(), name=defaults.DEFAULT_VARIABLE)

    @cached_method
    def _q_expansion_zero(self):
        """
        Return the `q`-expansion of the modular form 0.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M._q_expansion_zero()
            0
            sage: M._q_expansion_zero() == M._q_expansion_ring()(0)
            True
        """
        return self._q_expansion_ring()(0)

    def _q_expansion(self, element, prec):
        """
        Take an element of ``self`` (specified as a list, tuple, or vector),
        and return the corresponding `q`-expansion.

        EXAMPLES::

            sage: m = ModularForms(Gamma0(23),2); m
            Modular Forms space of dimension 3 for Congruence Subgroup Gamma0(23) of weight 2 over Rational Field
            sage: m.basis()
            [q - q^3 - q^4 + O(q^6),
             q^2 - 2*q^3 - q^4 + 2*q^5 + O(q^6),
             1 + 12/11*q + 36/11*q^2 + 48/11*q^3 + 84/11*q^4 + 72/11*q^5 + O(q^6)]
            sage: m._q_expansion([1,2,0], 5)
            q + 2*q^2 - 5*q^3 - 3*q^4 + O(q^5)
        """
        return self.ambient_module()._q_expansion(element, prec)

    def __add__(self, right):
        """
        If ``self`` and ``right`` live inside the same ambient module, return the
        sum of the two spaces (as modules).

        EXAMPLES::

            sage: N = CuspForms(44,2) ; ls = [N.submodule([N(u) for u in x.q_expansion_basis(20)]) for x in N.modular_symbols().decomposition()]; ls
            [Modular Forms subspace of dimension 1 of Modular Forms space of dimension 9 for Congruence Subgroup Gamma0(44) of weight 2 over Rational Field,
            Modular Forms subspace of dimension 3 of Modular Forms space of dimension 9 for Congruence Subgroup Gamma0(44) of weight 2 over Rational Field]

        ::

            sage: N1 = ls[0] ; N2 = ls[1]
            sage: N1 + N2 # indirect doctest
            Modular Forms subspace of dimension 4 of Modular Forms space of dimension 9 for Congruence Subgroup Gamma0(44) of weight 2 over Rational Field
        """
        from sage.modular.modform.submodule import ModularFormsSubmodule
        if self.ambient_module() != right.ambient_module():
            raise ArithmeticError(("Sum of %s and %s not defined because " +
                                    "they do not lie in a common ambient space.") %
                                   (self, right))
        if self.is_ambient():
            return self
        if right.is_ambient():
            return right
        V = self.free_module() + right.free_module()
        return ModularFormsSubmodule(self.ambient_module(), V)

    def _has_natural_inclusion_map_to(self, right):
        """
        Return true if there is a natural inclusion map from modular forms
        in ``self`` to modular forms in right.

        INPUT:

        - ``self``, ``right`` -- spaces of modular forms

        OUTPUT: ``True`` if ``self`` embeds in ``right``, ``False`` otherwise

        TODO: Barring a few trivial cases, this only works in the case that
        right.is_ambient() returns True.

        EXAMPLES::

            sage: N = ModularForms(6,4) ; S = N.cuspidal_subspace()

        ::

            sage: N._has_natural_inclusion_map_to(S)
            Traceback (most recent call last):
            ...
            NotImplementedError

        ::

            sage: S._has_natural_inclusion_map_to(N)
            True

        ::

            sage: M = ModularForms(11,2)
            sage: N._has_natural_inclusion_map_to(M)
            False
        """
        if not right.group().is_subgroup(self.group()):
            return False
        if right.character() is None:
            # It's the full Gamma_1(N).
            return True
        if right.is_ambient():
            e = self.character()
            f = right.character()
            return f.parent()(e) == f
        raise NotImplementedError

    def _coerce_map_from_(self, from_par):
        """
        Code to make ModularFormsSpace work well with coercion framework.

        EXAMPLES::

            sage: M = ModularForms(22,2)
            sage: M.has_coerce_map_from(M.cuspidal_subspace())
            True
            sage: M.has_coerce_map_from(ModularForms(22,4))
            False
        """
        if isinstance(from_par, ModularFormsSpace):
            if from_par.ambient() == self:
                return True
            if self.is_ambient() and self.group().is_subgroup(from_par.group()) and self.weight() == from_par.weight():
                return True

        return False

    def _element_constructor_(self, x, check=True):
        """
        Try to coerce ``x`` into ``self``. If ``x`` is a vector of length
        ``self.dimension()``, interpret it as a list of coefficients for
        ``self.basis()`` and return that linear combination. If ``x`` is a power
        series, it tries to determine whether or not ``x`` lives in ``self``. If
        so, it returns ``x`` as an element of M, and throws an error if not.

        EXAMPLES::

            sage: M = ModularForms(13,4)
            sage: M.dimension()
            5

        ::

            sage: M([1,2,3,4,5])
            4 + 6*q + 47*q^2 + 143*q^3 + 358*q^4 + 630*q^5 + O(q^6)

        ::

            sage: M([1,3])
            Traceback (most recent call last):
            ...
            TypeError: entries must be a list of length 5

        ::

            sage: R = M._q_expansion_ring()
            sage: M(R([0,1,0,0,0,-2,-4,-2,-12]).add_bigoh(9))
            q - 2*q^5 + O(q^6)

        ::

            sage: M.set_precision(9)
            sage: M(R([0,1,0,0,0,-2,-4,-2,-12]).add_bigoh(9))
            q - 2*q^5 - 4*q^6 - 2*q^7 - 12*q^8 + O(q^9)

        Note that one only needs coefficients up to self.sturm_bound() to
        determine the form::

            sage: M(R([0,1,0,0,0,-2,-4,-2,-12]).add_bigoh(8))
            q - 2*q^5 - 4*q^6 - 2*q^7 - 12*q^8 + O(q^9)

        ::

            sage: M(R([0,1,1,0,0,0,-4,-2,-12]).add_bigoh(9))
            Traceback (most recent call last):
            ...
            ValueError: q-expansion does not correspond to a form in self

        ::

            sage: S = CuspForms(1,12) ; R = PowerSeriesRing(QQ,'q') ; q = R.0
            sage: f = q+O(q^2) ; S(f)
            q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 + O(q^6)
            sage: f = q+2*q^2+O(q^3) ; S(f)
            Traceback (most recent call last):
            ...
            ValueError: q-expansion does not correspond to a form in self
            sage: f = q-24*q^2+O(q^3) ; S(f)
            q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 + O(q^6)

        Test that :issue:`13156` is fixed::

            sage: R.<q> = QQ[[]]
            sage: ModularForms(1, 12)(R(0))
            0
            sage: ModularForms(1, 12)(R(1))
            Traceback (most recent call last):
            ...
            TypeError: unable to create modular form from exact nonzero polynomial

            sage: E = ModularForms(3,12).cuspidal_subspace()
            sage: f = E.gens()[0]
            sage: g = f - f
            sage: g.is_old()
            True

        Test that :issue:`32168` is fixed::

            sage: M0 = ModularForms(Gamma0(8), 10)
            sage: M1 = ModularForms(Gamma1(8), 10)
            sage: f = M0.0; g = M1.0
            sage: f + g
            2*q + O(q^6)
            sage: M1(f)
            q + O(q^6)

        ::

            sage: M = ModularForms(22,2) ; S = CuspForms(22,2)
            sage: sum(S.basis())
            q + q^2 - q^3 - 4*q^4 + q^5 + O(q^6)
            sage: sum(S.basis() + M.basis())
            1 + 3*q + 3*q^2 + 2*q^3 - 7*q^4 + 8*q^5 + O(q^6)
            sage: M.coerce(S.basis()[0])
            q - q^3 - 2*q^4 + q^5 + O(q^6)

        ::

            sage: M = ModularForms(Gamma0(22)) ; N = ModularForms(Gamma0(44))
            sage: M.basis()[0]
            q - q^3 - 2*q^4 + q^5 + O(q^6)
            sage: N(M.basis()[0])
            q - q^3 - 2*q^4 + q^5 + O(q^6)

        TESTS::

            sage: M = ModularForms(13, 4)
            sage: M(M([1, 2, 3, 4, 5]), check=True)
            4 + 6*q + 47*q^2 + 143*q^3 + 358*q^4 + 630*q^5 + O(q^6)
        """
        if isinstance(x, ModularFormElement):

            if not check:
                from copy import copy
                f = copy(x)
                f._set_parent(self)
                return f

            if x.parent().ambient() == self:
                return self(x.element())

            return self(x.q_expansion(self._q_expansion_module().degree()))

        if isinstance(x, PowerSeries):
            if x.prec() == PlusInfinity():
                if x == 0:
                    return self.element_class(self, self.free_module().zero())
                raise TypeError("unable to create modular form from exact nonzero polynomial")
            W = self._q_expansion_module()
            if W.degree() <= x.prec():
                try:
                    x_potential = W.coordinates(x.padded_list(W.degree()))
                except ArithmeticError:
                    raise ValueError("q-expansion does not correspond to a form in self")
                x_potential = self.free_module().linear_combination_of_basis(x_potential)
                x_potential = self.element_class(self, x_potential)
                for i in range(int(W.degree()), x.prec()):
                    if x_potential[i] != x[i]:
                        raise ValueError("q-expansion does not correspond to a form in self")
                return x_potential
            raise TypeError("q-expansion needed to at least precision %s" % W.degree())

        return self.element_class(self, self.free_module()(x, check))

    def _pushout_(self, other):
        r"""
        Implement the pushout of ``self`` and ``other``.

        INPUT:

        - ``other`` -- ``ModularFormSpace`` or a ``ModularFormRing``

        OUTPUT: if ``self`` and ``other`` have the same groups and base rings, then this method returns
        ``self`` if the weights of the two spaces are equal, otherwise it returns a ``ModularFormsRing``.


        TESTS::

            sage: e4 = ModularForms(1,4).0; e6 = ModularForms(1,6).0;
            sage: M = ModularFormsRing(1)
            sage: e4 + e6
            2 - 264*q - 14472*q^2 - 116256*q^3 - 515208*q^4 - 1545264*q^5 + O(q^6)
            sage: (e4 + e6).parent()
            Ring of Modular Forms for Modular Group SL(2,Z) over Rational Field
            sage: (M(e4)*e6).parent()
            Ring of Modular Forms for Modular Group SL(2,Z) over Rational Field
            sage: f = ModularForms(5,12).0
            sage: f+e4
            Traceback (most recent call last):
            ...
            TypeError: unsupported operand parent(s) for +: 'Modular Forms space of dimension 7 for Congruence Subgroup Gamma0(5) of weight 12 over Rational Field' and 'Modular Forms space of dimension 1 for Modular Group SL(2,Z) of weight 4 over Rational Field'
        """
        from .ring import ModularFormsRing
        if isinstance(other, ModularFormsSpace):
            if self.group() == other.group() and self.base_ring() == other.base_ring():
                if self.weight() == other.weight():
                    return self
                return ModularFormsRing(self.group(), base_ring=self.base_ring())
        if isinstance(other, ModularFormsRing) and other.has_coerce_map_from(self):
            return other

    def __richcmp__(self, x, op):
        """
        Compare ``self`` and ``x``.

        For spaces of modular forms, we order first by signature, then by
        dimension, and then by the ordering on the underlying free
        modules.

        EXAMPLES::

            sage: N = ModularForms(6,4) ; S = N.cuspidal_subspace()
            sage: S < N
            True
            sage: N > S
            True
            sage: N == N
            True
            sage: M = ModularForms(11,2)
            sage: N < M
            True
            sage: M > N
            True
        """
        from sage.modular.modform.constructor import canonical_parameters as params

        if self is x:
            return rich_to_bool(op, 0)
        if not isinstance(x, ModularFormsSpace):
            return NotImplemented

        left_ambient = self.ambient()
        right_ambient = x.ambient()
        lx = params(left_ambient.character(), left_ambient.level(),
                    left_ambient.weight(), left_ambient.base_ring())
        rx = params(right_ambient.character(), right_ambient.level(),
                    right_ambient.weight(), right_ambient.base_ring())
        if lx != rx:
            return richcmp_not_equal(lx, rx, op)
        if self.is_ambient() or x.is_ambient():
            return richcmp(self.dimension(), x.dimension(), op)
        return self.free_module()._echelon_matrix_richcmp(x.free_module(), op)

    def span_of_basis(self, B):
        """
        Take a set B of forms, and return the subspace of ``self`` with B as a
        basis.

        EXAMPLES::

            sage: N = ModularForms(6,4) ; N
            Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field

        ::

            sage: N.span_of_basis([N.basis()[0]])
            Modular Forms subspace of dimension 1 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field

        ::

            sage: N.span_of_basis([N.basis()[0], N.basis()[1]])
            Modular Forms subspace of dimension 2 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field

        ::

            sage: N.span_of_basis( N.basis() )
            Modular Forms subspace of dimension 5 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
        """
        from .submodule import ModularFormsSubmoduleWithBasis
        W = self._q_expansion_module()
        F = self.free_module()
        prec = W.degree()
        C = [F.linear_combination_of_basis(W.coordinates(f.padded_list(prec))) for f in B]
        S = F.span_of_basis(C)
        return ModularFormsSubmoduleWithBasis(self.ambient(), S)

    span = span_of_basis

    def __submodule_from_subset_of_basis(self, x):
        """
        Return the submodule of ``self`` generated by the elements of x.

        EXAMPLES::

            sage: N = ModularForms(6,4)
            sage: N._ModularFormsSpace__submodule_from_subset_of_basis( [0,2] )
            Vector space of degree 5 and dimension 2 over Rational Field
            Basis matrix:
            [1 0 0 0 0]
            [0 0 1 0 0]
        """
        V = self.free_module()
        return V.submodule([V.gen(i) for i in x], check=False)

    def _compute_hecke_matrix_prime(self, p, prec=None):
        """
        Compute the matrix of the Hecke operator `T_p` acting on ``self``.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M._compute_hecke_matrix_prime(2)
            [-2  0]
            [ 0  3]

        ::

            sage: M = ModularForms(11,2)
            sage: M2 = M.span([M.0 + M.1])
            sage: M2.hecke_matrix(2)
            Traceback (most recent call last):
            ...
            ArithmeticError: vector is not in free module
        """
        if prec is None:
            # Initial guess -- will increase if need be.
            # We add on a few dimensions, so we are likely to
            # detect non-invariant subspaces (if they accidentally occur).
            prec = p*self.dimension() + 8
        try:
            cur, _ = self.__q_expansion_basis
        except AttributeError:
            pass
        else:
            prec = max(prec, cur)
        B = self.q_expansion_basis(prec)
        eps = self.character()
        if eps is None:
            raise NotImplementedError
        try:
            return hecke_operator_on_qexp.hecke_operator_on_basis(B, p,
                       self.weight(), eps, already_echelonized=False)
        except ValueError:
            # Double the precision.
            return self._compute_hecke_matrix_prime(p, prec=2 * prec + 1)

    def _compute_hecke_matrix(self, n):
        """
        Compute the matrix of the Hecke operator `T_n` acting on ``self``.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M._compute_hecke_matrix(6)
            [ 2  0]
            [ 0 12]

        ::

            sage: M = ModularForms(11,2)
            sage: M2 = M.span([M.0 + M.1])
            sage: M2.hecke_matrix(2)
            Traceback (most recent call last):
            ...
            ArithmeticError: vector is not in free module

        We check that :issue:`10450` is fixed::

            sage: M = CuspForms(Gamma1(22), 2).new_submodule()  # long time (3s on sage.math, 2011)
            sage: M.hecke_matrix(3)  # long time
            [ 0 -2  3  0]
            [ 0 -3  5 -1]
            [ 1 -1  0 -1]
            [ 0 -2  3 -1]
            sage: M.hecke_matrix(9)  # long time
            [ 3  3 -4 -4]
            [ 2  6 -9 -4]
            [ 0  3 -2 -1]
            [ 3  2 -7  0]
        """
        # For spaces with character, we calculate a basis of q-expansions and
        # use that. For Gamma1 and GammaH spaces, we would need to compute
        # diamond operators, which is quite slow; so we just compute on the
        # whole space and restrict.

        # TODO: If we know the subspace of the modular *symbols* space to which
        # this modular forms space corresponds, then that might give a quicker
        # way of doing this step.

        if hasattr(self, '_compute_q_expansion_basis') and self.character() is not None:
            return hecke.HeckeModule_generic._compute_hecke_matrix(self, n)

        # Try to avoid doing unnecessary computations where possible.
        if self.is_cuspidal():
            M = self.ambient().cuspidal_submodule().hecke_matrix(n).block_sum(zero_matrix(self.base_ring(), self.ambient().eisenstein_submodule().rank()))
        elif self.is_eisenstein():
            M = zero_matrix(self.base_ring(), self.ambient().cuspidal_submodule().rank()).block_sum(self.ambient().eisenstein_submodule().hecke_matrix(n))
        else:
            M = self.ambient().hecke_matrix(n)
        return M.restrict(self.free_module(), check=(gcd(n, self.level()) > 1))

    @cached_method
    def basis(self):
        """
        Return a basis for ``self``.

        EXAMPLES::

            sage: MM = ModularForms(11,2)
            sage: MM.basis()
            [q - 2*q^2 - q^3 + 2*q^4 + q^5 + O(q^6),
             1 + 12/5*q + 36/5*q^2 + 48/5*q^3 + 84/5*q^4 + 72/5*q^5 + O(q^6)]
        """
        return Sequence([self.element_class(self, x)
                         for x in self.free_module().basis()],
                        immutable=True, cr=True)

    def gen(self, n):
        """
        Return the `n`-th generator of ``self``.

        EXAMPLES::

            sage: N = ModularForms(6,4)
            sage: N.basis()
            [q - 2*q^2 - 3*q^3 + 4*q^4 + 6*q^5 + O(q^6),
             1 + O(q^6),
             q - 8*q^4 + 126*q^5 + O(q^6),
             q^2 + 9*q^4 + O(q^6),
             q^3 + O(q^6)]

        ::

            sage: N.gen(0)
            q - 2*q^2 - 3*q^3 + 4*q^4 + 6*q^5 + O(q^6)

        ::

            sage: N.gen(4)
            q^3 + O(q^6)

        ::

            sage: N.gen(5)
            Traceback (most recent call last):
            ...
            ValueError: Generator 5 not defined
        """
        try:
            return self.basis()[int(n)]
        except IndexError:
            raise ValueError("Generator %s not defined" % n)

    def gens(self) -> tuple:
        """
        Return a complete set of generators for ``self``.

        EXAMPLES::

            sage: N = ModularForms(6,4)
            sage: N.gens()
            (q - 2*q^2 - 3*q^3 + 4*q^4 + 6*q^5 + O(q^6),
             1 + O(q^6),
             q - 8*q^4 + 126*q^5 + O(q^6),
             q^2 + 9*q^4 + O(q^6),
             q^3 + O(q^6))
        """
        return tuple(self.basis())

    def sturm_bound(self, M=None):
        r"""
        For a space M of modular forms, this function returns an integer B
        such that two modular forms in either ``self`` or M are equal if and
        only if their `q`-expansions are equal to precision B (note that this
        is 1+ the usual Sturm bound, since `O(q^\mathrm{prec})` has
        precision ``prec``). If M is none, then M is set equal to ``self``.

        EXAMPLES::

            sage: S37=CuspForms(37,2)
            sage: S37.sturm_bound()
            8
            sage: M = ModularForms(11,2)
            sage: M.sturm_bound()
            3
            sage: ModularForms(Gamma1(15),2).sturm_bound()
            33

            sage: CuspForms(Gamma1(144), 3).sturm_bound()
            3457
            sage: CuspForms(DirichletGroup(144).1^2, 3).sturm_bound()
            73
            sage: CuspForms(Gamma0(144), 3).sturm_bound()
            73

        REFERENCES:

        - [Stu1987]_

        NOTE:

        Kevin Buzzard pointed out to me (William Stein) in Fall 2002 that
        the above bound is fine for Gamma1 with character, as one sees by
        taking a power of `f`. More precisely, if
        `f\cong 0\pmod{p}` for first `s` coefficients, then
        `f^r = 0 \pmod{p}` for first `s r` coefficients.
        Since the weight of `f^r` is
        `r \text{weight}(f)`, it follows that if
        `s \geq` the Sturm bound for `\Gamma_0` at
        weight(f), then `f^r` has valuation large enough to be
        forced to be `0` at `r\cdot` weight(f) by Sturm
        bound (which is valid if we choose `r` right). Thus
        `f \cong 0 \pmod{p}`. Conclusion: For `\Gamma_1`
        with fixed character, the Sturm bound is *exactly* the same as for
        `\Gamma_0`. A key point is that we are finding
        `\ZZ[\varepsilon]` generators for the Hecke algebra
        here, not `\ZZ`-generators. So if one wants
        generators for the Hecke algebra over `\ZZ`, this
        bound is wrong.

        This bound works over any base, even a finite field. There might be
        much better bounds over `\QQ`, or for comparing two
        eigenforms.
        """
        if M is not None:
            raise NotImplementedError
        if self.__sturm_bound is None:
            G = self.group()
            from sage.modular.arithgroup.congroup_gamma1 import Gamma1_class
            if isinstance(G, Gamma1_class) and self.character() is not None:
                from sage.modular.arithgroup.congroup_gamma0 import (
                    Gamma0_constructor as Gamma0,
                )
                G = Gamma0(self.level())
            # the +1 below is because O(q^prec) has precision prec.
            self.__sturm_bound = G.sturm_bound(self.weight())+1
        return self.__sturm_bound

    def cuspidal_submodule(self):
        """
        Return the cuspidal submodule of ``self``.

        EXAMPLES::

            sage: N = ModularForms(6,4) ; N
            Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
            sage: N.eisenstein_subspace().dimension()
            4

        ::

            sage: N.cuspidal_submodule()
            Cuspidal subspace of dimension 1 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field

        ::

            sage: N.cuspidal_submodule().dimension()
            1

        We check that a bug noticed on :issue:`10450` is fixed::

            sage: M = ModularForms(6, 10)
            sage: W = M.span_of_basis(M.basis()[0:2])
            sage: W.cuspidal_submodule()
            Modular Forms subspace of dimension 2 of Modular Forms space of dimension 11 for Congruence Subgroup Gamma0(6) of weight 10 over Rational Field
        """
        try:
            if self.__is_cuspidal:
                return self
        except AttributeError:
            pass
        if self.is_ambient():
            raise NotImplementedError("ambient modular forms spaces must override cuspidal_submodule")
        C = self.ambient_module().cuspidal_submodule()
        S = self.intersection(C)
        if S.dimension() < self.dimension():
            self.__is_cuspidal = False
        else:
            assert S.dimension() == self.dimension()
            self.__is_cuspidal = True
        S.is_eisenstein.set_cache(S.dimension() == 0)
        S.__is_cuspidal = True
        return S

    def cuspidal_subspace(self):
        """
        Synonym for cuspidal_submodule.

        EXAMPLES::

            sage: N = ModularForms(6,4) ; N
            Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
            sage: N.eisenstein_subspace().dimension()
            4

        ::

            sage: N.cuspidal_subspace()
            Cuspidal subspace of dimension 1 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field

        ::

            sage: N.cuspidal_submodule().dimension()
            1
        """
        return self.cuspidal_submodule()

    def is_cuspidal(self) -> bool:
        r"""
        Return ``True`` if this space is cuspidal.

        EXAMPLES::

            sage: M = ModularForms(Gamma0(11), 2).new_submodule()
            sage: M.is_cuspidal()
            False
            sage: M.cuspidal_submodule().is_cuspidal()
            True
        """
        return (self.cuspidal_submodule() == self)

    @cached_method
    def is_eisenstein(self) -> bool:
        r"""
        Return ``True`` if this space is Eisenstein.

        EXAMPLES::

            sage: M = ModularForms(Gamma0(11), 2).new_submodule()
            sage: M.is_eisenstein()
            False
            sage: M.eisenstein_submodule().is_eisenstein()
            True
        """
        return (self.eisenstein_submodule() == self)

    def new_submodule(self, p=None):
        """
        Return the new submodule of ``self``.

        If `p` is specified, return the `p`-new submodule of ``self``.

        .. NOTE::

            This function should be overridden by all derived classes.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: M = sage.modular.modform.space.ModularFormsSpace(Gamma0(11), 2, DirichletGroup(1)[0], base_ring=QQ); M.new_submodule()
            Traceback (most recent call last):
            ...
            NotImplementedError: computation of new submodule not yet implemented
        """
        raise NotImplementedError("computation of new submodule not yet implemented")

    def new_subspace(self, p=None):
        """
        Synonym for new_submodule.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: M = sage.modular.modform.space.ModularFormsSpace(Gamma0(11), 2, DirichletGroup(1)[0], base_ring=QQ); M.new_subspace()
            Traceback (most recent call last):
            ...
            NotImplementedError: computation of new submodule not yet implemented
        """
        return self.new_submodule(p)

    def eisenstein_series(self):
        """
        Compute the Eisenstein series associated to this space.

        .. NOTE::

           This function should be overridden by all derived classes.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: M = sage.modular.modform.space.ModularFormsSpace(Gamma0(11), 2, DirichletGroup(1)[0], base_ring=QQ); M.eisenstein_series()
            Traceback (most recent call last):
            ...
            NotImplementedError: computation of Eisenstein series in this space not yet implemented
        """
        raise NotImplementedError("computation of Eisenstein series in this space not yet implemented")

    def decomposition(self):
        """
        This function returns a list of submodules `V(f_i,t)`
        corresponding to newforms `f_i` of some level dividing the
        level of self, such that the direct sum of the submodules equals
        self, if possible. The space `V(f_i,t)` is the image under
        `g(q)` maps to `g(q^t)` of the intersection with
        `R[[q]]` of the space spanned by the conjugates of
        `f_i`, where `R` is the base ring of ``self``.

        TODO: Implement this function.

        EXAMPLES::

            sage: M = ModularForms(11,2); M.decomposition()
            Traceback (most recent call last):
            ...
            NotImplementedError
        """
        raise NotImplementedError

    def newforms(self, names=None):
        """
        Return all newforms in the cuspidal subspace of ``self``.

        EXAMPLES::

            sage: CuspForms(18,4).newforms()
            [q + 2*q^2 + 4*q^4 - 6*q^5 + O(q^6)]
            sage: CuspForms(32,4).newforms()
            [q - 8*q^3 - 10*q^5 + O(q^6), q + 22*q^5 + O(q^6), q + 8*q^3 - 10*q^5 + O(q^6)]
            sage: CuspForms(23).newforms('b')
            [q + b0*q^2 + (-2*b0 - 1)*q^3 + (-b0 - 1)*q^4 + 2*b0*q^5 + O(q^6)]
            sage: CuspForms(23).newforms()
            Traceback (most recent call last):
            ...
            ValueError: Please specify a name to be used when generating names for generators of Hecke eigenvalue fields corresponding to the newforms.
        """
        M = self.modular_symbols(sign=1)
        factors = M.cuspidal_subspace().new_subspace().decomposition()
        large_dims = [X.dimension() for X in factors if X.dimension() != 1]
        if large_dims and names is None:
            raise ValueError("Please specify a name to be used when generating names for generators of Hecke eigenvalue fields corresponding to the newforms.")
        elif names is None:
            # In this case, we don't need a variable name, so insert
            # something to get passed along below
            names = 'a'
        return [Newform(self, factors[i], names=names + str(i))
                for i in range(len(factors))]

    @cached_method
    def eisenstein_submodule(self):
        """
        Return the Eisenstein submodule for this space of modular forms.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M.eisenstein_submodule()
            Eisenstein subspace of dimension 1 of Modular Forms space of dimension 2 for Congruence Subgroup Gamma0(11) of weight 2 over Rational Field

        We check that a bug noticed on :issue:`10450` is fixed::

            sage: M = ModularForms(6, 10)
            sage: W = M.span_of_basis(M.basis()[0:2])
            sage: W.eisenstein_submodule()
            Modular Forms subspace of dimension 0 of Modular Forms space of dimension 11 for Congruence Subgroup Gamma0(6) of weight 10 over Rational Field
        """
        if self.is_eisenstein.cache is True:
            return self

        if self.is_ambient():
            raise NotImplementedError("ambient modular forms spaces must override eisenstein_submodule")
        A = self.ambient_module().eisenstein_submodule()
        E = self.intersection(A)
        if E.dimension() < self.dimension():
            self.is_eisenstein.set_cache(False)
        else:
            assert E.dimension() == self.dimension()
            self.is_eisenstein.set_cache(True)
        E.__is_cuspidal = (E.dimension() == 0)
        E.is_eisenstein.set_cache(True)
        return E

    def eisenstein_subspace(self):
        """
        Synonym for :meth:`eisenstein_submodule`.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: M.eisenstein_subspace()
            Eisenstein subspace of dimension 1 of Modular Forms space of dimension 2 for Congruence Subgroup Gamma0(11) of weight 2 over Rational Field
        """
        return self.eisenstein_submodule()

    def embedded_submodule(self):
        """
        Return the underlying module of ``self``.

        EXAMPLES::

            sage: N = ModularForms(6,4)
            sage: N.dimension()
            5

        ::

            sage: N.embedded_submodule()
            Vector space of dimension 5 over Rational Field
        """
        return self.free_module()

# intersect method commented out since it is a duplicate of the intersection method in sage.modular.hecke.submodule
# -- David Loeffler, 2009-04-30
#
#    def intersect(self, right):
#        """
#        If self and right live in the same ambient module, return the
#        intersection of self and right (as submodules).
#
#        EXAMPLES::
#
#            sage: N = ModularForms(6,4); S = N.cuspidal_subspace()
#
#        ::
#
#            sage: N.intersect(S)
#            Modular Forms subspace of dimension 1 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
#
#        ::
#
#            sage: S.intersect(N)
#            Modular Forms subspace of dimension 1 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
#
#        ::
#
#            sage: S.intersect(N.eisenstein_subspace())
#            Modular Forms subspace of dimension 0 of Modular Forms space of dimension 5 for Congruence Subgroup Gamma0(6) of weight 4 over Rational Field
#        """
#        from sage.modular.modform.all import ModularForms
#        if self.ambient_module() != right.ambient_module():
#            raise ArithmeticError("Intersection of %s and %s not defined." %
#                                  (self, right))
#        V = self.embedded_submodule().intersection(right.embedded_submodule())
#        return ModularForms(self.ambient_module(),V)
#        return self.span([ self(b) for b in V.basis() ])

#    def _key(self):
#        if self.is_ambient():
#            return self.__key
#        return self.__ambient

    def level(self):
        """
        Return the level of ``self``.

        EXAMPLES::

            sage: M = ModularForms(47,3)
            sage: M.level()
            47
        """
        return self.group().level()

    def modular_symbols(self, sign=0):
        """
        Return the space of modular symbols corresponding to ``self`` with the
        given sign.

        .. NOTE::

            This function should be overridden by all derived classes.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: M = sage.modular.modform.space.ModularFormsSpace(Gamma0(11), 2, DirichletGroup(1)[0], base_ring=QQ); M.modular_symbols()
            Traceback (most recent call last):
            ...
            NotImplementedError: computation of associated modular symbols space not yet implemented
        """
        raise NotImplementedError("computation of associated modular symbols space not yet implemented")

    def find_in_space(self, f, forms=None, prec=None, indep=True):
        """
        INPUT:

        - ``f`` -- a modular form or power series

        - ``forms`` -- (default: ``None``) a specific list of
          modular forms or `q`-expansions

        - ``prec`` -- if forms are given, compute with them to
          the given precision

        - ``indep`` -- boolean (default: ``True``); whether the given list
          of forms are assumed to form a basis

        OUTPUT: list of numbers that give f as a linear combination of
        the basis for this space or of the given forms if
        independent=True.

        .. NOTE::

           If the list of forms is given, they do *not* have to be in
           ``self``.

        EXAMPLES::

            sage: M = ModularForms(11,2)
            sage: N = ModularForms(10,2)
            sage: M.find_in_space( M.basis()[0] )
            [1, 0]

        ::

            sage: M.find_in_space( N.basis()[0], forms=N.basis() )
            [1, 0, 0]

        ::

            sage: M.find_in_space( N.basis()[0] )
            Traceback (most recent call last):
            ...
            ArithmeticError: vector is not in free module
        """
        if forms is None or (forms == []):
            B = self._q_expansion_module()
            V = B.ambient_module()
            n = B.degree()
        else:
            if not isinstance(forms, (list, tuple)):
                raise TypeError("forms must be a list or tuple")
            if prec is None:
                n = forms[0].parent().prec()
            else:
                n = prec
            V = self.base_ring()**n
            w = [V(g.padded_list(n)) for g in forms]
            if indep:
                B = V.span_of_basis(w)
            else:
                B = V.span(w)
        if isinstance(f, PowerSeries) and f.prec() < n:
            raise ValueError("you need at least %s terms of precision" % n)
        x = V(f.padded_list(n))
        return B.coordinates(x)


def contains_each(V, B):
    """
    Determine whether or not V contains every element of B. Used here
    for linear algebra, but works very generally.

    EXAMPLES::

        sage: contains_each = sage.modular.modform.space.contains_each
        sage: contains_each( range(20), prime_range(20) )
        True
        sage: contains_each( range(20), range(30) )
        False
    """
    return all(b in V for b in B)
